<?php
/**
 * Magento
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@magento.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade Magento to newer
 * versions in the future. If you wish to customize Magento for your
 * needs please refer to http://www.magento.com for more information.
 *
 * @category    Mage
 * @package     Mage_Checkout
 * @copyright  Copyright (c) 2006-2019 Magento, Inc. (http://www.magento.com)
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */

/**
 * One page checkout order review
 *
 * @category    Mage
 * @package     Mage_Checkout
 * @author      Magento Core Team <core@magentocommerce.com>
 */
class Jmango360_Japi_Block_Checkout_Onepage_Review_Info extends Mage_Checkout_Block_Onepage_Review_Info
{
    protected function getQuote()
    {
        return Mage::getSingleton('checkout/session')->getQuote();
    }

    public function getBillingAddress()
    {
        return $this->getQuote()->getBillingAddress();
    }

    public function getShippingAddress()
    {
        return $this->getQuote()->getShippingAddress();
    }

    public function getShippingMethod()
    {
        return $this->getShippingAddress();
    }

    public function getPaymentMethod()
    {
        return $this->getQuote()->getPayment()->getMethodInstance();
    }

    /**
     * Get quote shipping price including tax
     * @return float
     */
    public function getShippingPriceInclTax()
    {
        $inclTax = $this->getShippingAddress()->getShippingInclTax();
        return $this->formatPrice($inclTax);
    }

    public function getShippingPriceExclTax()
    {
        return $this->formatPrice($this->getShippingAddress()->getShippingAmount());
    }

    public function formatPrice($price)
    {
        return $this->getQuote()->getStore()->formatPrice($price);
    }

    /**
     * Get address lines
     *
     * @param $address
     * @return string
     */
    public function getAddressLines($address)
    {
        $lines = array();
        $lines[] = sprintf('%s %s %s %s %s',
            $address->getPrefix(),
            $address->getFirstname(),
            $address->getMiddlename(),
            $address->getLastname(),
            $address->getSuffix()
        );

        if (in_array($address->getCountryId(), array('US', 'AU'))) {
            $lines[] = join(' ', $address->getStreet());
            $lines[] = sprintf('%s %s %s', $address->getCity(), $address->getRegion(), $address->getPostcode());
            $lines[] = Mage::app()->getLocale()->getCountryTranslation($address->getCountryId());
        } elseif (in_array($address->getCountryId(), array('CN', 'FR', 'IL'))) {
            $lines[] = join(' ', $address->getStreet());
            $lines[] = sprintf('%s %s %s', $address->getPostcode(), $address->getCity(), $address->getRegion());
            $lines[] = Mage::app()->getLocale()->getCountryTranslation($address->getCountryId());
        } elseif (in_array($address->getCountryId(), array('NL', 'ES', 'IT', 'DE', 'PT', 'SE', 'DK'))) {
            $lines[] = join(' ', $address->getStreet());
            $lines[] = sprintf('%s %s', $address->getPostcode(), $address->getCity());
            if ($address->getRegion()) {
                $lines[] = $address->getRegion();
            }
            $lines[] = Mage::app()->getLocale()->getCountryTranslation($address->getCountryId());
        } else {
            $lines[] = join(' ', $address->getStreet());
            if ($address->getCity()) {
                $lines[] = sprintf('%s', $address->getCity());
            }
            $lines[] = sprintf('%s %s', $address->getPostcode(), $address->getRegion());
            $lines[] = Mage::app()->getLocale()->getCountryTranslation($address->getCountryId());
        }

        $lines[] = sprintf('<a href="tel:%s">%s</a>', $address->getTelephone(), $address->getTelephone());

        return join('<br/>', $lines);
    }
}
