<?php

/**
 * Copyright 2016 JMango360
 */
if (Mage::helper('core')->isModuleEnabled('Nedstars_Checkout') && @class_exists('Nedstars_Checkout_Block_Onepage_Shipping')) {
    include('Shipping_Nedstars.php');
} else {
    include('Shipping_Jmango360.php');
}

class Jmango360_Japi_Block_Checkout_Onepage_Shipping extends Jmango360_Japi_Block_Checkout_Onepage_Shipping_Abstract
{
    public function __construct()
    {
        if (Mage::helper('core')->isModuleEnabled('Massamarkt_Core')) {
            $this->setTemplate('japi/checkout/onepage/massamarkt/shipping.phtml');
        }
        parent::__construct();
    }

    public function getAddress()
    {
        if (is_null($this->_address)) {
            $this->_address = $this->getQuote()->getShippingAddress();
        }

        return $this->_address;
    }

    public function setV2SelectedAddress($selectedAddress)
    {
        $this->_address = $selectedAddress;
    }

    public function getV2SelectedAddress()
    {
        return $this->_address;
    }

    public function getAddressListV2()
    {
        if ($this->isCustomerLoggedIn()) {
            $addresses = $this->getCustomer()->getAddresses();
            if ($this->getQuote()->getShippingAddress()) {
                $billingAddressId = $this->getQuote()->getShippingAddress()->getCustomerAddressId();
                foreach ($addresses as $address) {
                    if ($billingAddressId == $address->getId()) {
                        $address->setData('isShipping', true);
                        $this->setV2SelectedAddress($address);
                    }
                }
            }
            return $addresses;
        } else {
            return $this->getQuote()->getBillingAddress()->setData('isShipping', true);
        }
    }

    public function isAddressListEmpty()
    {
        if ($this->isCustomerLoggedIn()) {
            return count($this->getCustomer()->getAddresses()) == 0 && !$this->getQuote()->getShippingAddress();
        } else {
            return !$this->getQuote()->getShippingAddress();
        }
    }

    /**
     * MPLUGIN-3261: Not support edit address anymore
     *
     * @param $type
     * @return string
     */
    public function _getAddressesHtmlSelect($type)
    {
        $html = parent::getAddressesHtmlSelect($type);

        if (Mage::app()->getRequest()->getModuleName() == 'japi' && $html) {
            $editLink = sprintf('<div class="japi-address-edit"><a href="#" class="japi-address-edit-btn">%s</a></div>',
                $this->__('Edit')
            );

            $html .= $editLink;
        }

        return $html;
    }

    public function getAddressListJson()
    {
        $output = array();

        if ($this->isCustomerLoggedIn()) {
            $shippingAddress = $this->getQuote()->getShippingAddress();
            $shippingAddressId = $shippingAddress->getCustomerAddressId();

            if (!$shippingAddressId) {
                if ($this->getCustomer()->getDefaultShippingAddress()) {
                    $shippingAddressId = $this->getCustomer()->getDefaultShippingAddress()->getId();
                }
            }

            $addresses = $this->getCustomer()->getAddresses();
            foreach ($addresses as $address) {
                $data = $this->_getAddressData($address);

                if ($address->getId() == $shippingAddressId) {
                    $data['is_selected'] = true;
                } else {
                    $data['is_selected'] = false;
                }

                $output[] = $data;
            }

            /*if (!$billingAddressId) {
                $data = $this->_getAddressData($billingAddress);
                $data['is_selected'] = true;
                $output[] = $data;
            }*/
        }

        return $output;
    }

    protected function _getAddressData($address)
    {
        $attributes = array(
            'entity_id', 'email', 'prefix', 'firstname', 'middlename', 'lastname', 'suffix', 'company', 'street', 'postcode',
            'city', 'region', 'region_id', 'country', 'country_id', 'telephone', 'fax'
        );

        $output = array();

        foreach ($attributes as $attribute) {
            if ($attribute == 'street') {
                $output[$attribute] = explode("\n", $address->getData($attribute));
            } elseif ($attribute == 'country') {
                if ($address->getData('country')) {
                    $output[$attribute] = $address->getData('country');
                } else {
                    $country = Mage::getSingleton('directory/country')->loadByCode($address->getData('country_id'));
                    if ($country->getId()) {
                        $output[$attribute] = $country->getName();
                    }
                }
            } else {
                $output[$attribute] = $address->getData($attribute);
            }
        }

        return $output;
    }
}