<?php

/**
 * Copyright 2017 JMango360
 */
class Jmango360_Japi_Block_Js extends Mage_Core_Block_Template
{
    /**
     * Order ID
     *
     * @var int
     */
    protected $orderId;

    /**
     * Additional params
     *
     * @var array
     */
    protected $_additionalParams;

    protected function _construct()
    {
        parent::_construct();
        $this->setTemplate('japi/js.phtml');
    }

    /**
     * Only append this script if:
     * - Not in internal browser mobile
     * - Not in JAPI route
     * - After place order
     *
     * @return bool
     */
    public function isShow()
    {
        /**
         * MPLUGIN-3055
         */
        /** @var Mage_Core_Model_Session $coreSession */
        $coreSession = Mage::getSingleton('core/session');
        if ($coreSession->getData('jm360_check_popup')) {
            /** @var Mage_Core_Model_Cookie $cookie */
            $cookie = Mage::getSingleton('core/cookie');
            if ($cookie->get('skip_back_to_app')) {
                $cookie->delete('skip_back_to_app');
                $coreSession->unsetData('jm360_check_popup');

                return false;
            }
        } else {
            return false;
        }

        /* @var $httpHelper Mage_Core_Helper_Http */
        $httpHelper = Mage::helper('core/http');
        if (strpos($httpHelper->getHttpUserAgent(), 'JM360-Mobile') !== false) {
            return false;
        }

        if (strpos(Mage::app()->getRequest()->getPathInfo(), 'japi/') !== false) {
            return false;
        }

        return true;
    }

    /**
     * Set order ID
     *
     * @param $orderId
     * @return $this
     */
    public function setOrderId($orderId)
    {
        $this->orderId = $orderId;
        return $this;
    }

    /**
     * Get order ID
     *
     * @return int
     */
    public function getOrderId()
    {
        return $this->orderId;
    }

    /**
     * Set additional params
     *
     * @param array $params
     * @return $this
     */
    public function setAdditionalParams($params = array())
    {
        $this->_additionalParams = $params;
        return $this;
    }

    /**
     * Get additional params
     *
     * @return array
     */
    public function getAdditionalParams()
    {
        return $this->_additionalParams;
    }

    /**
     * Return uri scheme with params for Android or iOS
     *
     * @param $uri string
     * @param $orderID string
     * @return string
     */
    public function getUriScheme($uri, $orderID = null)
    {
        if (!$uri) return '';

        $uri = trim($uri);
        if (!$orderID) $orderID = $this->getOrderId();
        if (!$orderID) return '';

        /** @var Mage_Sales_Model_Order $order */
        $order = Mage::getModel('sales/order');
        $order->loadByIncrementId($orderID);
        if (!$order->getId()) return '';

        $params = array(
            'orderId' => $orderID, // for android
            'transasction_id' => $orderID, // for iOS
            'currency' => $order->getOrderCurrencyCode(),
            'price_incl' => $order->getGrandTotal()
        );

        if (is_array($this->_additionalParams)) {
            $params = array_merge($params, $this->_additionalParams);
        }

        if (strpos($uri, 'intent://') === 0) { // android uri
            $androidParams = array();
            foreach ($params as $key => $value) {
                $androidParams[] = sprintf('S.%s=%s', $key, $value);
            }

            return str_replace(';end', sprintf(';%s;end', join(';', $androidParams)), $uri);
        } else { // iOS uri
            $iOSParams = array();
            foreach ($params as $key => $value) {
                $iOSParams[] = sprintf('%s=%s', $key, $value);
            }

            return strpos($uri, '?') !== false
                ? sprintf('%s&%s', $uri, join('&', $iOSParams))
                : sprintf('%s?%s', $uri, join('&', $iOSParams));
        }
    }
}
