<?php
/**
 * Copyright 2020 JMango360
 */

/**
 * Class Jmango360_Japi_Helper_Hook
 */
class Jmango360_Japi_Helper_Hook extends Mage_Core_Helper_Abstract
{
    const XML_PATH_HOOKS = 'japi/jmango_rest_stock_settings/hooks';

    /**
     * Get all hooks
     *
     * @param bool $isActive
     */
    public function getHooks($isActive = true)
    {
        try {
            $cache = Mage::app()->getCache();
            $hooks = json_decode($cache->load(self::XML_PATH_HOOKS), true);
            if (!$hooks) {
                $hooks = json_decode(Mage::getStoreConfig(self::XML_PATH_HOOKS), true);
            }

            $hooks = is_array($hooks) ? $hooks : [];
            $count = count($hooks);
            for ($i = $count - 1; $i >= 0; $i--) {
                if (!is_array($hooks[$i])) {
                    unset($hooks[$i]);
                }
                if ($isActive && !$hooks[$i]['is_active']) {
                    unset($hooks[$i]);
                }
            }

            return $hooks;
        } catch (\Exception $e) {
            return [];
        }
    }

    /**
     * Get active hooks by scope
     *
     * @param string $type
     * @return bool
     */
    public function getActiveHooks($type)
    {
        $hooks = $this->getHooks(true);
        foreach ($hooks as $hook) {
            if (!empty($hook['scope']) && $hook['scope'] === $type) {
                return true;
            }
        }

        return false;
    }

    /**
     * Save hook to storage
     *
     * @param array $hooks
     */
    public function saveHooks($hooks)
    {
        $config = Mage::app()->getConfig();
        $config->saveConfig(self::XML_PATH_HOOKS, json_encode($hooks));

        $cache = Mage::app()->getCache();
        $cache->save(json_encode($hooks), self::XML_PATH_HOOKS, array(Mage_Core_Model_Config::CACHE_TAG), null);
    }
}