<?php
/**
 * Copyright 2018 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Observer_Cache
 */
class Jmango360_Japi_Model_Observer_Cache
{
    /**
     * Clear cache
     *
     * @param $event
     * @throws
     */
    public function modelSaveAfter($event)
    {
        $model = $event->getEvent()->getObject();

        if (Mage::app()->useCache(Jmango360_Japi_Model_Server::JMANGO360_CACHE_TYPE)) {
            /** @var Mage_Core_Model_Cache $cache */
            $cache = Mage::getSingleton('core/cache');
            /** @var Jmango360_Japi_Helper_debug $logger */
            $logger = Mage::helper('japi/debug');

            if ($model instanceof Mage_Catalog_Model_Product && $model->getId()) {
                $productIds = array($model->getId());

                /**
                 * Get parent configurable product
                 */
                /** @var Mage_Catalog_Model_Product_Type_Configurable $productConfigurableType */
                $productConfigurableType = Mage::getSingleton('catalog/product_type_configurable');
                $configurableParentIds = $productConfigurableType->getParentIdsByChild($model->getId());
                $productIds = array_merge($productIds, $configurableParentIds);

                /**
                 * Get parent grouped product
                 */
                /** @var Mage_Catalog_Model_Product_Type_Grouped $productGroupedType */
                $productGroupedType = Mage::getSingleton('catalog/product_type_grouped');
                $groupedParentIds = $productGroupedType->getParentIdsByChild($model->getId());
                $productIds = array_merge($productIds, $groupedParentIds);

                /**
                 * Get parent bundle product
                 */
                /** @var Mage_Bundle_Model_Product_Type $productBundleType */
                $productBundleType = Mage::getSingleton('bundle/product_type');
                $bundleParentIds = $productBundleType->getParentIdsByChild($model->getId());
                $productIds = array_merge($productIds, $bundleParentIds);

                if (count(array_unique($productIds))) {
                    $this->cleanAPIProductsCache($productIds);
                }
            } elseif ($model instanceof Mage_Catalog_Model_Category) {
                if ($cache->clean('CATEGORY')) {
                    $logger->log(array('Clean' => 'CATEGORY'), 'japi_cache.log');
                }
            }
        }

        if ($model instanceof Mage_Sales_Model_Order && $model->getData('japi')) {
            /** @var Jmango360_Japi_Helper_Hook $hookHelper */
            $hookHelper = Mage::helper('japi/hook');
            if ($hookHelper->getActiveHooks(Jmango360_Japi_Model_Orchard_Sync::EVENT_ORDER_STATUS)) {
                /** @var Jmango360_Japi_Model_Orchard_Sync $orchard */
                $orchard = Mage::getSingleton('japi/orchard_sync');
                $orchard->addJob(array(
                    'type' => $orchard::EVENT_ORDER_STATUS,
                    'orderId' => $model->getId(),
                    'orderIncrementId' => $model->getIncrementId(),
                    'status' => $model->getStatus(),
                    'origStatus' => $model->getOrigData('status'),
                    'state' => $model->getState(),
                    'origState' => $model->getOrigData('state'),
                    'customerId' => $model->getCustomerId()
                ));
            }
        }
    }

    /**
     * Clear all cache after reindex
     *
     * @param $event
     */
    public function afterReindex($event)
    {
        /** @var Mage_Core_Model_Cache $cache */
        $cache = Mage::getSingleton('core/cache');
        /** @var Jmango360_Japi_Helper_debug $logger */
        $logger = Mage::helper('japi/debug');

        $cache->clean(Jmango360_Japi_Model_Server::JMANGO360_CACHE_TAG);
        $logger->log(array('Clean' => Jmango360_Japi_Model_Server::JMANGO360_CACHE_TAG), 'japi_cache.log');
    }

    /**
     * Clear related API cache for order items
     *
     * @param $event
     */
    public function checkoutSubmitAllAfter($event)
    {
        switch (Mage::getStoreConfig('japi/jmango_rest_developer_settings/order_cache_invalid')) {
            case Jmango360_Japi_Model_System_Config_Source_Cache_Order::MODE_ALWAYS:
                $quote = $event->getEvent()->getQuote();

                /**
                 * Collect all items in order
                 */
                $productIds = array();
                foreach ($quote->getAllItems() as $item) {
                    $productIds[$item->getProductId()] = $item->getProductId();
                    $children = $item->getChildrenItems();
                    if ($children) {
                        foreach ($children as $childItem) {
                            $productIds[$childItem->getProductId()] = $childItem->getProductId();
                        }
                    }
                }

                /**
                 * Clear products cache
                 */
                if (count($productIds)) {
                    $this->cleanAPIProductsCache($productIds);
                }
                break;
            case Jmango360_Japi_Model_System_Config_Source_Cache_Order::MODE_NEVER:
            default:
                break;
            case Jmango360_Japi_Model_System_Config_Source_Cache_Order::MODE_STOCK:
                $quote = $event->getEvent()->getQuote();

                /**
                 * Collect all items and qty in order
                 */
                $productIds = array();
                foreach ($quote->getAllItems() as $item) {
                    /** @var Mage_Sales_Model_Quote_Item $item */
                    $productId = $item->getProductId();
                    if (!$productId) continue;

                    $children = $item->getChildrenItems();
                    if ($children) {
                        foreach ($children as $childItem) {
                            /** @var Mage_Sales_Model_Quote_Item $childItem */
                            if ($this->_checkOutOfStock($childItem)) {
                                $productIds[] = $childItem->getProductId();
                                $productIds[] = $productId;
                            }
                        }
                    } else {
                        if ($this->_checkOutOfStock($item)) {
                            $productIds[] = $productId;
                            if ($item->getParentItemId()) {
                                $productIds[] = $item->getParentItem()->getProductId();
                            }
                        }
                    }
                }

                /**
                 * Clear products cache
                 */
                $productIds = array_unique($productIds);
                if (count($productIds)) {
                    $this->cleanAPIProductsCache($productIds);
                }
                break;
        }
    }

    /**
     * Check if product is out of stock after purchase
     *
     * @param Mage_Sales_Model_Quote_Item $quoteItem
     * @return bool
     */
    protected function _checkOutOfStock($quoteItem)
    {
        if ($quoteItem->getProduct()) {
            /** @var Mage_CatalogInventory_Model_Stock_Item $stockItem */
            $stockItem = $quoteItem->getProduct()->getStockItem();

            switch ($quoteItem->getProduct()->getTypeId()) {
                case Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE:
                case Mage_Catalog_Model_Product_Type::TYPE_GROUPED:
                case Mage_Catalog_Model_Product_Type::TYPE_BUNDLE:
                    return false;
                    break;
                case Mage_Catalog_Model_Product_Type::TYPE_SIMPLE:
                default:
                    return (float)$stockItem->getQty() - (float)$quoteItem->getTotalQty() <= 0;
                    break;
            }
        }

        return false;
    }

    /**
     * Clean API cache for products
     *
     * @param $productIds Array of product IDs
     */
    public function cleanAPIProductsCache($productIds)
    {
        if (!$productIds || empty($productIds)) {
            return;
        }

        /** @var Mage_Core_Model_Cache $cache */
        $cache = Mage::getSingleton('core/cache');
        /** @var Jmango360_Japi_Helper_debug $logger */
        $logger = Mage::helper('japi/debug');

        $tags = array();
        foreach ($productIds as $productId) {
            $tags[] = 'PRODUCT_' . $productId;
        }

        if (count($tags)) {
            if ($cache->clean($tags)) {
                $debug = array('Clean' => join(',', $tags));
                $logger->log($debug, 'japi_cache.log');
            }
        }
    }
}
