<?php
/**
 * Copyright 2017 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Observer_Catalog_Category
 */
class Jmango360_Japi_Model_Observer_Catalog_Category
{
    const EVENT_TMP = 'japi_event_tmp';
    const EMPTY_TMP = 'japi_empty_tmp';
    const CACHE_KEY = 'JAPI_CATALOG_CATEGORY_EVENT';
    const MAX_RERTY = 1;
    const EVENT_ADD = 1;
    const EVENT_UPDATE = 2;
    const EVENT_DELETE = 3;
    const EVENT_MOVE = 4;
    const URL_INT = 'https://merkel-api.jmango360.com/webhook/';
    const URL_UAT = 'https://kim-api.jmango360.com/webhook/';
    const URL_PRO = 'https://api-1480.jmango360.com/webhook/';

    /**
     * @var int Retry counter
     */
    protected $_retry = 0;

    /**
     * Before category save
     *
     * @param Varien_Event_Observer $observer
     */
    public function catalogCategorySaveBefore($observer)
    {
        $category = $observer->getEvent()->getCategory();
        if ($category instanceof Mage_Catalog_Model_Category) {
            if ($category->getId()) {
                $category->setData(self::EVENT_TMP, self::EVENT_UPDATE);
                $category->setData(self::EMPTY_TMP, $this->isCategoryEmpty($category));
            } else {
                $category->setData(self::EVENT_TMP, self::EVENT_ADD);
            }
        }
    }

    /**
     * After category save
     *
     * @param Varien_Event_Observer $observer
     */
    public function catalogCategorySaveAfter($observer)
    {
        $category = $observer->getEvent()->getCategory();
        if ($category instanceof Mage_Catalog_Model_Category) {
            /** @var Mage_Catalog_Model_Category $newCategory */
            $newCategory = Mage::getModel('catalog/category')->load($category->getId());
            if ($category->getData(self::EVENT_TMP) == self::EVENT_ADD) {
                $this->addJob($category->getId(), self::EVENT_ADD);
            } elseif ($category->getData(self::EVENT_TMP) == self::EVENT_UPDATE) {
                if ($this->isCategoryEmpty($newCategory)) {
                    $this->addJob($category->getId(), self::EVENT_DELETE);
                } else {
                    if ($category->getData(self::EMPTY_TMP)) {
                        $this->addJob($category->getId(), self::EVENT_ADD);
                    } else {
                        $this->addJob($category->getId(), self::EVENT_UPDATE);
                    }
                }
            }
        }
    }

    /**
     * Before category delete
     *
     * @param Varien_Event_Observer $observer
     */
    public function catalogCategoryDeleteBefore($observer)
    {
        $category = $observer->getEvent()->getCategory();
        if ($category instanceof Mage_Catalog_Model_Category) {
            $this->addJob($category->getId(), self::EVENT_DELETE);
        }
    }

    /**
     * After move category
     *
     * @param Varien_Event_Observer $observer
     */
    public function catalogCategoryMoveAfter($observer)
    {
        $category = $observer->getEvent()->getCategory();
        if ($category instanceof Mage_Catalog_Model_Category) {
            $category->setData('old_parent_id', $observer->getEvent()->getPrevParentId());
            $this->addJob($category, self::EVENT_MOVE);
        }
    }

    public function banAllProductCaches($event) {
        /* @var Jmango360_Japi_Helper_Data $japiHelper */
        $japiHelper = Mage::helper('japi');
        if ($japiHelper->isModuleEnabled('Nexcessnet_Turpentine')
            && Mage::helper('turpentine/varnish')->getVarnishEnabled()) {
            $japiHelper->_getVarnishAdmin()->flushUrl('./japi/rest_product.');
        }
    }

    /**
     * Store job to cache
     *
     * @param Mage_Catalog_Model_Category|int $categoryId
     * @param int $eventName
     */
    protected function addJob($categoryId, $eventName)
    {
        if (1) {
            $this->process($categoryId, $eventName);
        } else {
            $cache = Mage::app()->getCache();
            try {
                $data = json_decode($cache->load(self::CACHE_KEY));
            } catch (Exception $e) {
                $data = new stdClass();
            }

            $data->$categoryId = $eventName;

            try {
                $cache->save(json_encode($data), self::CACHE_KEY);
            } catch (Exception $e) {
                Mage::logException($e);
            }
        }
    }

    /**
     * Remove job from cache
     *
     * @param $categoryId
     */
    protected function removeJob($categoryId)
    {
        $cache = Mage::app()->getCache();
        try {
            $data = json_decode($cache->load(self::CACHE_KEY));
        } catch (Exception $e) {
            $data = new stdClass();
        }

        unset($data->$categoryId);

        try {
            $cache->save(json_encode($data), self::CACHE_KEY);
        } catch (Exception $e) {
            Mage::logException($e);
        }
    }

    /**
     * Auto-update function
     */
    public function autoUpdate()
    {
        if (!Mage::getStoreConfigFlag('japi/jmango_rest_catalog_settings/auto_update_catalog')) {
            return;
        }

        $cache = Mage::app()->getCache();
        try {
            $data = json_decode($cache->load(self::CACHE_KEY));
        } catch (Exception $e) {
            $data = new stdClass();
        }
        foreach (get_object_vars($data) as $cid => $event) {
            if ($cid && $event) {
                if ($this->process($cid, $event)) {
                    $this->removeJob($cid);
                }
                break;
            }
        }
    }

    /**
     * @param $categoryId
     * @param $eventName
     * @return bool
     */
    protected function process($categoryId, $eventName)
    {
        if (!$categoryId || !$eventName) return false;

        $store = Mage::app()->getRequest()->getParam('store');
        if ($store) {
            $this->_process($categoryId, $eventName, $store);
        } else {
            $category = $this->loadCategory($categoryId);
            if (!$category) return false;
            $pathIds = $category->getPathIds();
            $stores = Mage::app()->getStores();
            foreach ($stores as $s) {
                /** @var $s Mage_Core_Model_Store */
                $rootCategoryId = $s->getRootCategoryId();
                if (in_array($rootCategoryId, $pathIds)) {
                    $this->_process($categoryId, $eventName, $s->getId());
                }
            }
        }

        return true;
    }

    /**
     * @param $categoryId
     * @param $eventName
     * @param int $store
     * @return bool
     */
    protected function _process($categoryId, $eventName, $store)
    {
        if (!$this->isEnable($store)) return false;
        if (!$categoryId || !$eventName) return false;

        if ($store) {
            /** @var Mage_Core_Model_App_Emulation $emulator */
            $emulator = Mage::getSingleton('core/app_emulation');
            $env = $emulator->startEnvironmentEmulation($store);
        }

        try {
            switch ($eventName) {
                case self::EVENT_ADD:
                    $this->processAdd($categoryId);
                    break;
                case self::EVENT_UPDATE:
                    $this->processUpdate($categoryId);
                    break;
                case self::EVENT_DELETE:
                    $this->processDelete($categoryId);
                    break;
                case self::EVENT_MOVE:
                    $this->processMove($categoryId);
                    break;
            }
        } catch (Exception $e) {
            Mage::logException($e);
        }

        if ($store && isset($emulator) && isset($env)) {
            $emulator->stopEnvironmentEmulation($env);
        }

        return true;
    }

    /**
     * @param $categoryId
     * @return bool
     */
    protected function processMove($categoryId)
    {
        $category = $this->loadCategory($categoryId);
        if (!$category) return false;

        $url = $this->getUrl() . 'mgt/catalog/move';
        $params = array_merge($this->getDefaultParams(), array(
            'id' => $category->getId(),
            'name' => $category->getName(),
            'parentId' => $this->getParentId($category),
            'oldParent' => $category->getData('old_parent_id'),
            'size' => $category->getProductCount(),
            'image' => $this->getImageUrl($category)
        ));

        try {
            $result = $this->query('POST', $url, $params);
            if (isset($result['code']) && $result['code'] == 200) {
                return true;
            }
        } catch (Exception $e) {
            $this->log('ERROR: ' . $e->getMessage());
        }

        return false;
    }

    /**
     * @param $categoryId
     * @param $noCheckEmpty
     * @return bool
     */
    protected function processAdd($categoryId, $noCheckEmpty = false)
    {
        $category = $this->loadCategory($categoryId);
        if (!$category) return false;

        $parentId = $this->getParentId($category);
        if ($parentId != -1) {
            $parent = $this->loadCategory($parentId);
            if ($parent->getId()) {
                if ($parent->getChildrenCount() == 1) {
                    return $this->processAdd($parent->getId(), true);
                }
            }
        }

        if ($this->isCategoryDisabled($category, true)) {
            return false;
        }

        if (!$noCheckEmpty && $this->isCategoryEmpty($category)) {
            return false;
        }

        $url = $this->getUrl() . 'mgt/catalog/add';
        $params = array_merge($this->getDefaultParams(), array(
            'id' => $category->getId(),
            'name' => $category->getName(),
            'disable' => $this->isCategoryDisabled($category, true),
            'parentId' => $this->getParentId($category),
            'image' => $this->getImageUrl($category),
            'size' => $category->getProductCount()
        ));

        try {
            $result = $this->query('POST', $url, $params);
            if (isset($result['code']) && $result['code'] == 200) {
                return true;
            }
        } catch (Exception $e) {
            $this->log('ERROR: ' . $e->getMessage());
        }

        return false;
    }

    /**
     * @param $categoryId
     * @return bool
     */
    protected function processUpdate($categoryId)
    {
        $category = $this->loadCategory($categoryId);
        if (!$category) return false;

        if ($this->isCategoryEmpty($category)) {
            return $this->processDelete($categoryId);
        }

        $url = $this->getUrl() . 'mgt/catalog/update';
        $params = array_merge($this->getDefaultParams(), array(
            'id' => $category->getId(),
            'name' => $category->getName(),
            'disable' => $this->isCategoryDisabled($category),
            'parentId' => $this->getParentId($category),
            'size' => $category->getProductCount(),
            'image' => $this->getImageUrl($category)
        ));

        try {
            $result = $this->query('POST', $url, $params);
            if (isset($result['code']) && $result['code'] == 200) {
                return true;
            }
        } catch (Exception $e) {
            $this->log('ERROR: ' . $e->getMessage());
        }

        return false;
    }

    /**
     * @param $categoryId
     * @return bool
     */
    protected function processDelete($categoryId)
    {
        $category = $this->loadCategory($categoryId);
        if (!$category) return false;

        $url = $this->getUrl() . 'mgt/catalog/delete';
        $params = array_merge($this->getDefaultParams(), array(
            'id' => $category->getId(),
            'parentId' => $this->getParentId($category)
        ));

        try {
            $result = $this->query('POST', $url, $params);
            if (isset($result['code']) && $result['code'] == 200) {
                return true;
            }
        } catch (Exception $e) {
            $this->log('ERROR: ' . $e->getMessage());
        }

        return false;
    }

    /**
     * Get parent of category, -1 if category is root
     *
     * @param Mage_Catalog_Model_Category $category
     * @return int
     */
    protected function getParentId($category)
    {
        if ($category->getLevel() == 2) return -1;
        else return $category->getParentId();
    }

    /**
     * Get category thumbnail url
     *
     * @param $category
     * @return string
     */
    protected function getImageUrl($category)
    {
        /** @var Jmango360_Japi_Model_Rest_Catalog_Category_Tree $model */
        $model = Mage::getModel('japi/rest_catalog_category_tree');
        return $model->getCategoryThumbnailUrl($category);
    }


    /**
     * Return category object
     *
     * @param $categoryId
     * @return Mage_Catalog_Model_Category|null
     */
    protected function loadCategory($categoryId)
    {
        if (!$categoryId) return null;
        if ($categoryId instanceof Mage_Catalog_Model_Category) {
            $oldParentId = $categoryId->getData('old_parent_id');
            $categoryId = $categoryId->getId();
        }
        if (($storeId = Mage::app()->getRequest()->getParam('store', false)) === false) {
            try {
                $storeId = Mage::app()->getStore()->getId();
            } catch (Exception $e) {
                return null;
            }
        }
        /** @var Mage_Catalog_Model_Category $category */
        $category = Mage::getModel('catalog/category');
        $category->setStoreId($storeId)->load($categoryId);
        if (!empty($oldParentId)) {
            $category->setData('old_parent_id', $oldParentId);
        }

        return $category->getId() ? $category : null;
    }

    /**
     * Check if category dont have children and products
     *
     * @param Mage_Catalog_Model_Category $category
     * @return bool
     */
    protected function isCategoryEmpty($category)
    {
        return !$category->hasChildren() && !$category->getProductCount();
    }

    /**
     * Check if category should disabled on Orchard
     *
     * @param Mage_Catalog_Model_Category $category
     * @param bool $isAdd
     * @return bool
     */
    protected function isCategoryDisabled($category, $isAdd = false)
    {
        if (!$category) return true;
        if (!$category->getData('is_active')) return true;
        if (!$category->getData('include_in_menu') && !Mage::getStoreConfig('japi/jmango_rest_catalog_settings/include_all_active')) {
            return true;
        }
        if ($isAdd) return false;
        return !$category->hasChildren() && !$category->getProductCount();
    }

    /**
     * Get API KEY integration
     */
    protected function getDefaultParams()
    {
        return array(
            'apiKey' => Mage::getStoreConfig('japi/jmango_rest_api/apikey'),
            'storeUrl' => Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_LINK, true),
            'storeId' => Mage::app()->getStore()->getId()
        );
    }

    /**
     * Send request to Orchard
     *
     * @param string $method
     * @param string $url
     * @param array $params
     * @return array
     * @throws Exception
     */
    protected function query($method = 'GET', $url, $params = array())
    {
        $curl = curl_init();

        $this->log(sprintf('%s: %s', $method, $url));

        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_ENCODING, "");
        curl_setopt($curl, CURLOPT_MAXREDIRS, 1);
        curl_setopt($curl, CURLOPT_TIMEOUT, 5);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);

        switch ($method) {
            case 'POST':
                $paramsStr = json_encode($params);
                curl_setopt($curl, CURLOPT_POST, 1);
                curl_setopt($curl, CURLOPT_POSTFIELDS, $paramsStr);
                $this->log(sprintf('DATA: %s', $paramsStr));
                break;
        }

        // Header
        $headers = array(
            "Cache-Control: no-cache",
            "Content-Type: application/json",
            "Accept-Language: en",
            "JM360_APP_VERSION: x.x.x",
            "JM360_VERSION: x.x.x",
            "JM360_APP_ID: x.x.x",
            "JM360_APP_TYPE: 1",
            "X-Powered-By: JM360-Mobile"
        );
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);

        // Verify ssl certificate and host
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);

        // Response
        $response = curl_exec($curl);
        $err = curl_error($curl);
        if ($err) {
            $this->log('ERROR: ' . $err);
            throw new Exception($err);
        }
        $responseHttpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $this->log(sprintf('%s: %s', $responseHttpCode, $response));

        // Close connection
        curl_close($curl);

        // Rety
        try {
            $responseData = json_decode($response, true);
            if (isset($responseData['error']['code'])) {
                if ($this->_retry < self::MAX_RERTY) {
                    $this->log('Retry ' . ++$this->_retry);
                    return $this->query($method, $url, $params);
                }
            }
        } catch (Exception $e) {
        }

        return array('code' => $responseHttpCode, 'body' => $response);
    }

    /**
     * Log request response
     *
     * @param $data
     */
    protected function log($data)
    {
        Mage::log($data, null, 'japi_category_autoupdate.log');
    }

    /**
     * Get Orchard URL
     *
     * @return string|null
     */
    protected function getUrl()
    {
        $env = Mage::getStoreConfig('japi/jmango_rest_catalog_settings/environment');
        switch ($env) {
            case '2':
                return self::URL_INT;
                break;
            case '1':
                return self::URL_UAT;
                break;
            default:
                return self::URL_PRO;
                break;
        }
    }

    /**
     * Check if auto-update is active
     *
     * @param $store
     * @return bool
     */
    protected function isEnable($store)
    {
        return Mage::getStoreConfig('japi/jmango_rest_catalog_settings/auto_update_catalog', $store);
    }
}