<?php
/**
 * Copyright 2015 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Response
 */
class Jmango360_Japi_Model_Response extends Mage_Core_Controller_Response_Http
{
    /**
     * Character set which must be used in response
     */
    const RESPONSE_CHARSET = 'utf-8';

    /**
     * Default message types
     */
    const MESSAGE_TYPE_SUCCESS = 'success';
    const MESSAGE_TYPE_ERROR = 'error';
    const MESSAGE_TYPE_WARNING = 'warning';

    protected $_skipToken = false;

    /**
     * Store raw data before convert to response
     * Used for cache
     *
     * @var mixed
     */
    protected $_rawBody;

    /**
     * Get raw body
     *
     * @return mixed
     */
    public function getRawBody()
    {
        return $this->_rawBody;
    }

    /**
     * Set header appropriate to specified MIME type
     *
     * @param string $mimeType MIME type
     * @return $this
     */
    public function setMimeType($mimeType)
    {
        $this->setHeader('Content-Type', "{$mimeType}; charset=" . self::RESPONSE_CHARSET, true);
        return $this;
    }

    public function setSkipToken($flag)
    {
        $this->_skipToken = (bool)$flag;
    }

    public function render(array $data)
    {
        $this->_rawBody = $data;

        if (!$this->_skipToken) {
            $data['token'] = Mage::getSingleton('japi/server')->getToken();
        }

        /**
         * Support GGMGastro, return current shipping country to all API requests
         */
        if (Mage::helper('core')->isModuleEnabled('Flagbit_LanguageSelector')) {
            /** @var Mage_Directory_Model_Country $country */
            $country = Mage::helper('languageselector')->getCurrentShippingCountry();
            $this->setHeader('Japi-Shipping-Country', $country->getIso2Code());
            $this->setHeader('Japi-App-Name', 'ggmgastro');
        }

        $this->setMimeType($this->getRenderer()->getMimeType())
            ->setBody($this->getRenderer()->render($data));
    }

    /**
     * Get API reponse renderer
     *
     * @return Jmango360_Japi_Model_Renderer_Json
     */
    protected function getRenderer()
    {
        /** @var Jmango360_Japi_Model_Renderer_Json $render */
        $render = Mage::getModel('japi/renderer_json');
        return $render;
    }
}
