<?php

class Jmango360_Japi_Model_Rest_Catalog_Category_Assignedproducts extends Mage_Catalog_Model_Category
{
    /**
     * Initialize requested category object
     *
     * @return Mage_Catalog_Model_Category
     * @throws Jmango360_Japi_Exception
     */
    public function getAssignedProducts()
    {
        $category = $this->_initCategory();

        /* @var $helper Jmango360_Japi_Helper_Product */
        $helper = Mage::helper('japi/product');

        if (is_object($category) && $category->getId()) {
            $block = $this->_getListBlock();
            $productCollection = $block->getLayer()->getProductCollection();
            /* @var $resource Mage_Core_Model_Resource */
            $resource = Mage::getSingleton('core/resource');
            $productCollection->getSelect()
                ->join(
                    array('p' => $resource->getTableName('catalog/product')),
                    sprintf(
                        'e.entity_id = p.entity_id AND p.type_id IN (%s)',
                        join(',', array('"simple"', '"configurable"', '"grouped"', '"bundle"'))
                    ),
                    null
                );

            if (!$productCollection->getSize()) {
                $data['message'] = $helper->__('No products found.');
            }

            $data['filters'] = $this->_getFilters();
            $helper->addPageSettings($productCollection);
            $data['toolbar_info'] = $helper->getToolBarInfo($productCollection);
            /**
             * Add group by product's ID to collection
             */
            if (version_compare(Mage::getVersion(), '1.9.2.1', '>')) {
                $productCollection->getSelect()->group('e.entity_id');
            }

            $productCollection->clear();
            $data['products'] = $helper->convertProductCollectionToApiResponse($productCollection);
        } else {
            throw new Jmango360_Japi_Exception(
                $helper->__('Category not found.'),
                Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR
            );
        }

        return $data;
    }

    /**
     * @return Jmango360_Japi_Block_Catalog_Layer_View|Mage_Catalog_Block_Navigation
     */
    protected function _getLayerBlock()
    {
        /**
         * MPLUGIN-2270: Support GoMage_Navigation
         */
        if (Mage::helper('core')->isModuleEnabled('GoMage_Navigation')) {
            Mage::app()->getLayout()->createBlock('page/html_head', 'head');
            return Mage::helper('japi')->getBlock('GoMage_Navigation_Block_Layer_View');
        }

        /**
         * MPLUGIN-1601: Support Amasty_Shopby
         */
        if (Mage::helper('core')->isModuleEnabled('Amasty_Shopby')) {
            if (strpos($_SERVER['HTTP_HOST'], 'dangerousminds') !== false) {
                return Mage::helper('japi')->getBlock('Amasty_Shopby_Block_Catalog_Layer_View_Top');
            }
            return Mage::helper('japi')->getBlock('Amasty_Shopby_Block_Catalog_Layer_View');
        }

        /**
         * MPLUGIN-2801: Support Mana_Filters
         */
        if (Mage::helper('core')->isModuleEnabled('Mana_Filters')) {
            $manaBlock = Mage::helper('japi')->getBlock('Mana_Filters_Block_View');
            return $manaBlock->delayedPrepareLayout();
        }

        //skip non-anchor category
        $category = Mage::registry('current_category');
        if ($category->getIsAnchor()) {
            return Mage::helper('japi')->getBlock('Jmango360_Japi_Block_Catalog_Layer_View');
        } else {
            return Mage::helper('japi')->getBlock('Mage_Catalog_Block_Navigation');
        }
    }

    /**
     * @return Jmango360_Japi_Block_Catalog_Product_List
     */
    protected function _getListBlock()
    {
        return Mage::helper('japi')->getBlock('Jmango360_Japi_Block_Catalog_Product_List');
    }

    /**
     * @return Mage_Catalog_Block_Layer_State
     */
    protected function _getStateBlock()
    {
        return Mage::helper('japi')->getBlock('Mage_Catalog_Block_Layer_State');
    }

    /**
     * @return array
     */
    protected function _getFilters()
    {
        $data = array();

        /* @var $helper Jmango360_Japi_Helper_Data */
        $helper = Mage::helper('japi');
        $block = $this->_getLayerBlock();

        if ($helper->isModuleEnabled('GGMGastro_Catalog')) {
            return $data;
        } else if (Mage::helper('core')->isModuleEnabled('Amasty_Shopby')) {
            $filters = $this->_getAmastyFilters();
        } else {
            $filters = $block->getFilters();
        }

        if (!$filters || !is_array($filters)) return $data;

        foreach ($filters as $key => $filter) {
            /* @var $filter Mage_Catalog_Block_Layer_Filter_Abstract */
            if ($filter->getType() == 'catalog/layer_filter_category') {
                continue;
            }

            if ($filter->getAttributeModel()
                && array_key_exists($filter->getAttributeModel()->getAttributeCode(), Mage::app()->getRequest()->getParams())
            ) {
                continue;
            }

            if ($helper->isFilterEnabled($filter, $block) && $filter->getItemsCount()) {
                $filterData = $this->_filterToArray($filter);
                if (!empty($filterData['items'])) $data[] = $filterData;
            }
        }

        return $data;
    }

    /**
     * Converts Mage_Catalog_Block_Layer_Filter_Abstract into array
     *
     * @param $filter Mage_Catalog_Block_Layer_Filter_Abstract
     * @return array
     */
    protected function _filterToArray($filter)
    {
        $data = array();

        /**
         * MPLUGIN-3314 Filter - Website display Filter by Category and On Sale Filter
         * Update check amasty filter code (move to helper)
         */
        $attributeCode = Mage::helper('japi')->_getFilterCode($filter);

        if (!$attributeCode) return $data;

        if (strpos(Mage::getBaseUrl(), 'dangerousminds') !== false) {
            $data['name'] = $filter->getName();
        } elseif (strpos(Mage::getBaseUrl(), 'oswaldparfum') !== false) {
            if ($attributeCode == 'type_custom') {
                $data['name'] = Mage::helper('core')->__('Product Type');
            } else {
                $data['name'] = Mage::helper('japi')->__($filter->getName());
            }
        } else {
            $data['name'] = Mage::helper('japi')->__($filter->getName());
        }

        if (Mage::helper('core')->isModuleEnabled('Amasty_Shopby')) {
            if ($filter instanceof Amasty_Shopby_Block_Catalog_Layer_Filter_OnSale) {
                $data['code'] = 'onSale';
            } elseif ($filter instanceof Amasty_Shopby_Block_Catalog_Layer_Filter_New) {
                $data['code'] = 'new';
            } else {
                $data['code'] = $attributeCode;
            }
        } else {
            $data['code'] = $attributeCode;
        }

        if ($filter instanceof Amasty_Shopby_Block_Catalog_Layer_Filter_Attribute) {
            foreach ($filter->getItemsAsArray() as $item) {
                if (empty($item['id'])) {
                    foreach ($filter->getItems() as $oItem) {
                        if ($oItem->getLabel() == $item['label']) {
                            $item['id'] = $oItem->getValue();
                            break;
                        }
                    }
                }
                $data['items'][] = array(
                    'count' => $item['countValue'],
                    'label' => $item['label'],
                    'value' => $item['id'],
                    'url' => null
                );
            }
        } elseif ($filter instanceof Mage_Catalog_Block_Layer_Filter_Price) {
            if (Mage::helper('core')->isModuleEnabled('Amasty_Shopby') && class_exists('Amasty_Shopby_Model_Catalog_Layer_Filter_Price')) {
                if ((defined('Amasty_Shopby_Model_Catalog_Layer_Filter_Price::DT_DEFAULT') && $filter->hasDisplayType() == Amasty_Shopby_Model_Catalog_Layer_Filter_Price::DT_DEFAULT)
                    || (defined('Amasty_Shopby_Model_Catalog_Layer_Filter_Price::DT_DROPDOWN') && $filter->hasDisplayType() == Amasty_Shopby_Model_Catalog_Layer_Filter_Price::DT_DROPDOWN)
                ) {
                    foreach ($filter->getItems() as $item) {
                        $data['items'][] = $this->_itemToArray($item, $data['code']);
                    }
                }
            } else {
                foreach ($filter->getItems() as $item) {
                    $data['items'][] = $this->_itemToArray($item, $data['code']);
                }
            }
        } else {
            foreach ($filter->getItems() as $item) {
                $data['items'][] = $this->_itemToArray($item, $data['code']);
            }
        }

        if (strpos(Mage::getBaseUrl(), 'bloomfashion') !== false && $data['code'] == 'manufacturer') {
            usort($data['items'], array($this, "_compareByLabel"));
        }

        return $data;
    }

    /**
     * Compare two label values of array
     * @param $a
     * @param $b
     * @return int
     */
    protected function _compareByLabel($a, $b)
    {
        return strcmp($a["label"], $b["label"]);
    }

    /**
     * Converts array Mage_Catalog_Model_Layer_Filter_Item into array
     *
     * @param $item Mage_Catalog_Model_Layer_Filter_Item
     * @param $code
     * @return array
     */
    protected function _itemToArray($item, $code)
    {
        /**
         * MPLUGIN-1144:
         * Fix isse filter when installed "Gomage_Navigation"
         * and set Use Friendly URLs to "Yes" in Admin config of this extension
         */
        /* @var $helper Jmango360_Japi_Helper_Product */
        $helper = Mage::helper('japi/product');
        $_label = (string)$item->getLabel();
        $_value = (string)$item->getValue();
        if ($helper->isModuleEnabled('GoMage_Navigation')) {
            $_ignoreAttrCode = array('price');
            if (!in_array($code, $_ignoreAttrCode)) {
                $gomageHelper = Mage::helper('gomage_navigation');
                if ($gomageHelper->isFrendlyUrl()) {
                    $_value = $gomageHelper->formatUrlValue($_label, $_value);
                }
            }
        }

        $data = array();
        $data['count'] = (int)$item->getCount();
        $data['label'] = strip_tags($_label);
        $data['value'] = $_value;
        $data['url'] = null;

        return $data;
    }

    /**
     * Initialize requested category object
     *
     * @return Mage_Catalog_Model_Category
     * @throws Jmango360_Japi_Exception
     */
    protected function _initCategory()
    {
        /* @var $server Jmango360_Japi_Model_Server */
        $server = Mage::helper('japi')->getServer();
        /* @var $controller Jmango360_Japi_Rest_CatalogController */
        $controller = $server->getControllerInstance();

        Mage::dispatchEvent('catalog_controller_category_init_before', array('controller_action' => $controller));
        $categoryId = (int)Mage::helper('japi')->getRequest()->getParam('category_id', false);
        if (!$categoryId) {
            throw new Jmango360_Japi_Exception(
                Mage::helper('japi')->__('Category not found.'),
                Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR
            );
        }

        /* @var $category Mage_Catalog_Model_Category */
        $category = Mage::getModel('catalog/category')
            ->setStoreId(Mage::app()->getStore()->getId())
            ->load($categoryId);

        if (!Mage::helper('catalog/category')->canShow($category)) {
            throw new Jmango360_Japi_Exception(
                Mage::helper('japi')->__('Category not available.'),
                Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR
            );
        }

        Mage::getSingleton('catalog/session')->setLastVisitedCategoryId($category->getId());
        Mage::register('current_category', $category);
        Mage::register('current_entity_key', $category->getPath());

        try {
            Mage::dispatchEvent(
                'catalog_controller_category_init_after',
                array(
                    'category' => $category,
                    'controller_action' => $controller
                )
            );
        } catch (Mage_Core_Exception $e) {
            Mage::logException($e);
            return false;
        }

        return $category;
    }


    /**
     * Get filters of Amasty
     *
     * @return array
     */
    protected function _getAmastyFilters()
    {
        $filterResult = array();

        if (strpos(Mage::getUrl(), 'modaitalia.nl') === false) {
            $blockView = Mage::helper('japi')->getBlock('Amasty_Shopby_Block_Catalog_Layer_View');
            if ($blockView) {
                $leftFilters = $blockView->getFilters();
                if (is_array($leftFilters) && count($leftFilters) > 0) {
                    foreach ($leftFilters as $filter) {
                        if (!in_array($filter, $filterResult)) {
                            $filterResult[] = $filter;
                        }
                    }
                }
            }
        }

        $blockViewTop = Mage::helper('japi')->getBlock('Amasty_Shopby_Block_Catalog_Layer_View_Top');
        if ($blockViewTop) {
            $topFilters = $blockViewTop->getFilters();
            if (is_array($topFilters) && count($topFilters) > 0) {
                foreach ($topFilters as $filter) {
                    if (!in_array($filter, $filterResult)) {
                        $filterResult[] = $filter;
                    }
                }
            }

        }

        if (strpos(Mage::getUrl(), 'modaitalia.nl') === false) {
            $blockViewRight = Mage::helper('japi')->getBlock('Amasty_Shopby_Block_Catalog_Layer_View_Right');
            if ($blockViewRight) {
                $rightFilters = $blockViewRight->getFilters();
                if (is_array($rightFilters) && count($rightFilters) > 0) {
                    foreach ($rightFilters as $filter) {
                        if (!in_array($filter, $filterResult)) {
                            $filterResult[] = $filter;
                        }
                    }
                }
            }
        }

        return $filterResult;
    }
}
