<?php

class Jmango360_Japi_Model_Rest_Catalog_Search_Products extends Mage_CatalogSearch_Model_Query
{
    /**
     * Catalog Product collection
     *
     * @var Mage_CatalogSearch_Model_Resource_Fulltext_Collection
     */
    protected $_productCollection;

    /**
     * Get search result
     */
    public function getList()
    {
        /* @var $searchHelper Mage_CatalogSearch_Helper_Data */
        $searchHelper = Mage::helper('catalogsearch');
        /* @var $query Mage_CatalogSearch_Model_Query */
        $query = $searchHelper->getQuery();

        $query->setStoreId(Mage::app()->getStore()->getId());
        $data = array();
        if ($query->getQueryText() != '') {
            if ($searchHelper->isMinQueryLength()) {
                $query->setId(0)
                    ->setIsActive(1)
                    ->setIsProcessed(1);
            } else {
                if ($query->getId()) {
                    $query->setPopularity($query->getPopularity() + 1);
                } else {
                    $query->setPopularity(1);
                }

                $query->prepare();
            }

            /**
             * By calling this block the Magento standard template load search layout is simulated
             *   -- this way different implementations of search engines and params will be active
             *   -- without code changes
             */
            $helper = Mage::helper('japi/product');
            /* @var $helper Jmango360_Japi_Helper_Product */
            $block = $this->_getSearchLayerBlock();
            /* @var $productCollection Mage_Catalog_Model_Resource_Product_Collection */
            $productCollection = $block->getLayer()->getProductCollection();

            /* @var $resource Mage_Core_Model_Resource */
            $resource = Mage::getSingleton('core/resource');
            $productCollection->getSelect()->join(
                array('p' => $resource->getTableName('catalog/product')),
                sprintf(
                    'e.entity_id = p.entity_id AND p.type_id IN (%s)',
                    join(',', array('"simple"', '"configurable"', '"grouped"', '"bundle"'))
                ),
                null
            );

            if (!$productCollection->getSize()) {
                $data['message'] = $searchHelper->__('Your search returns no results.');
            }

            $data['filters'] = $this->_getFilters($block);

            if (strpos(Mage::getUrl(), 'wijnkooperijdelange.nl') !== false) {
                $amastyBlock = Mage::app()->getLayout()->createBlock('Amasty_Shopby_Block_Search_Layer');
                $data['filters'] = $this->_getFilters($amastyBlock);
            }

            $helper->addPageSettings($productCollection);
            $data['toolbar_info'] = $helper->getToolBarInfo($productCollection, true);
            /**
             * Add group by product's ID to collection
             */
            if (version_compare(Mage::getVersion(), '1.9.2.1', '>')) {
                $productCollection->getSelect()->group('e.entity_id');
            }
            $productCollection->clear();
            $data['products'] = $helper->convertProductCollectionToApiResponse($productCollection);

            $searchHelper->checkNotes();
            $messages = $searchHelper->getNoteMessages();
            if (!empty($messages)) {
                $data['message'] .= implode("\n", (array)$messages);
            }

            if (!$searchHelper->isMinQueryLength()) {
                $query->save();
            }
        } else {
            throw new Jmango360_Japi_Exception(Mage::helper('japi')->__('Query cannot be empty.'), Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR);
        }

        return $data;
    }

    /**
     * @return Jmango360_Japi_Block_Catalogsearch_Layer
     */
    protected function _getSearchLayerBlock()
    {
        Mage::app()->getRequest()->setModuleName('catalogsearch');

        /* @var $coreHelper Mage_Core_Helper_Data */
        $coreHelper = Mage::helper('core');

        if ($coreHelper->isModuleEnabled('Smile_ElasticSearch')) {
            return Mage::app()->getLayout()->createBlock('Smile_ElasticSearch_Block_Catalogsearch_Layer');
        } elseif ($coreHelper->isModuleEnabled('ElasticSearch_Asn')) {
            return Mage::app()->getLayout()->createBlock('ElasticSearch_Asn_Block_Catalog_Layer_Elasticsearch');
        } elseif ($coreHelper->isModuleEnabled('Amasty_Shopby')) {
            if (strpos(Mage::getUrl(), 'wijnkooperijdelange.nl') !== false) {
                return Mage::app()->getLayout()->createBlock('Jmango360_Japi_Block_Catalogsearch_Layer');
            } else {
                return Mage::app()->getLayout()->createBlock('Amasty_Shopby_Block_Search_Layer');
            }
        } else {
            return Mage::app()->getLayout()->createBlock('Jmango360_Japi_Block_Catalogsearch_Layer');
        }
    }

    protected function _getFilters($block = null)
    {
        $data = array();

        if (!$block) return $data;

        $filters = $block->getFilters();

        if (!$filters || !is_array($filters)) return $data;

        /* @var $helper Jmango360_Japi_Helper_Data */
        $helper = Mage::helper('japi');

        foreach ($filters as $key => $filter) {
            /* @var $filter Mage_Catalog_Block_Layer_Filter_Abstract */
            if (!$this->_shouldShow($filter)) {
                continue;
            }

            if ($filter->getItemsCount() && $helper->isFilterEnabled($filter, $block)) {
                $arrFilter = $this->_filterToArray($filter);
                if ($arrFilter != null) {
                    $data[] = $arrFilter;
                }
            }
        }

        return $data;
    }

    protected function _shouldShow(Mage_Catalog_Block_Layer_Filter_Abstract $filter)
    {
        /**
         * MPLUGIN-3314 Filter - Website display Filter by Category and On Sale Filter
         * Update check amasty filter code (move to helper)
         */
        $code = Mage::helper('japi')->_getFilterCode($filter);

        if (!$code) {
            return false;
        }

        if ($filter->getRequest()->getParam($code)) {
            if ($code != 'cat') {
                return false;
            }
        }

        /* @var $helper Mage_Core_Helper_Data */
        $helper = Mage::helper('core');

        if ($code == 'price') {
            if ($helper->isModuleEnabled('Smile_ElasticSearch')) {
                return false;
            }
        }

        if ($code == 'cat') {
            if ($helper->isModuleEnabled('Klevu_Search')) {
                return false;
            }
            if ($helper->isModuleEnabled('GGMGastro_Catalog')) {
                return false;
            }
        }

        return true;
    }

    /**
     * Converts Mage_Catalog_Model_Layer_Filter_Abstract into array
     *
     * @param Mage_Catalog_Block_Layer_Filter_Abstract $filter
     * @return array
     */
    protected function _filterToArray(Mage_Catalog_Block_Layer_Filter_Abstract $filter)
    {
        $data = array();
        $data['name'] = Mage::helper('japi')->__($filter->getName());

        /**
         * MPLUGIN-3314 Filter - Website display Filter by Category and On Sale Filter
         * add function to check code filter (move to helper)
         */
        $data['code'] = Mage::helper('japi')->_getFilterCode($filter);

        foreach ($filter->getItems() as $item) {
            if ((int)$item->getCount() > 0) {
                $data['items'][] = $this->_itemToArray($item, $data['code']);
            }
        }

        if (empty($data['items'])) {
            return null;
        }

        if (strpos($_SERVER['HTTP_HOST'], 'bloomfashion') !== false && $data['code'] == 'manufacturer') {
            usort($data['items'], array($this, "_compareByLabel"));
        }

        return $data;
    }

    /**
     * Compare two label values of array
     * @param $a
     * @param $b
     * @return int
     */
    protected function _compareByLabel($a, $b)
    {
        return strcmp($a["label"], $b["label"]);
    }


    /**
     * Converts array Mage_Catalog_Model_Layer_Filter_Item into array
     *
     * @param $item
     * @param $code
     * @return array
     */
    protected function _itemToArray($item, $code)
    {
        /**
         * MPLUGIN-1144:
         * Fix isse filter when installed "Gomage_Navigation"
         * and set Use Friendly URLs to "Yes" in Admin config of this extension
         */
        /* @var $helper Jmango360_Japi_Helper_Product */
        $helper = Mage::helper('japi/product');
        $_label = (string)$item->getLabel();
        $_value = (string)$item->getValue();
        if ($helper->isModuleEnabled('GoMage_Navigation')) {
            $_ignoreAttrCode = array('price');
            if (!in_array($code, $_ignoreAttrCode)) {
                $gomageHelper = Mage::helper('gomage_navigation');
                if ($gomageHelper->isFrendlyUrl()) {
                    $_value = $gomageHelper->formatUrlValue($_label, $_value);
                }
            }
        }

        $data = array();
        $data['count'] = (int)$item->getCount();
        $data['label'] = $_label;
        $data['value'] = $_value;

        return $data;
    }
}