<?php
/**
 * Copyright 2020 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Rest_Coupon
 */
class Jmango360_Japi_Model_Rest_Coupon extends Mage_Core_Model_Abstract
{
    public function dispatch()
    {
        $action = $this->_getRequest()->getAction();
        $operation = $this->_getRequest()->getOperation();

        switch ($action . $operation) {
            case 'create' . Jmango360_Japi_Model_Request::OPERATION_CREATE:
                $data = $this->_createCoupon();
                $this->_getResponse()->render($data);
                $this->_getResponse()->setHttpResponseCode(Jmango360_Japi_Model_Server::HTTP_OK);
                break;
            default:
                throw new Jmango360_Japi_Exception(Mage::helper('japi')->__('Resource method not implemented'), Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR);
                break;
        }
    }

    /**
     * @throws
     */
    protected function _createCoupon()
    {
        /** @var $model Mage_SalesRule_Model_Rule */
        $model = Mage::getModel('salesrule/rule');
        //Mage::dispatchEvent('adminhtml_controller_salesrule_prepare_save', array('request' => $this->getRequest()));
        $data = $this->_getRequest()->getParams();
        if (isset($data['rule']) && Mage::helper('adminhtml')->hasTags($data['rule'], array('attribute'), false)) {
            throw new Jmango360_Japi_Exception(Mage::helper('catalogrule')->__('Wrong rule specified'), Jmango360_Japi_Model_Request::HTTP_BAD_REQUEST);
        }
        $data = $this->_filterDates($data, array('from_date', 'to_date'));

        $validateResult = $model->validateData(new Varien_Object($data));
        if ($validateResult !== true) {
            $messages = array();
            foreach ($validateResult as $errorMessage) {
                $messages[] = $errorMessage;
            }

            throw new Jmango360_Japi_Exception(join("\n", $messages), Jmango360_Japi_Model_Request::HTTP_BAD_REQUEST);
        }

        $storeId = $this->_getRequest()->getParam('store_id');
        if ($storeId) {
            $store = Mage::app()->getStore($storeId);
            if ($store->getId() == $storeId) {
                $data['website_ids'] = array($store->getWebsiteId());
            }
        } else {
            $data['website_ids'] = array(Mage::app()->getStore()->getWebsiteId());
        }

        /* @var $collection Mage_Customer_Model_Resource_Group_Collection */
        $collection = Mage::getResourceModel('customer/group_collection');
        foreach ($collection as $group) {
            /* @var $group Mage_Customer_Model_Group */
            $data['customer_group_ids'][] = $group->getId();
        }

        if (empty($data['name'])) {
            $data['name'] = $data['coupon_code'];
        }
        if (empty($data['description'])) {
            $data['description'] = $data['coupon_code'];
        }
        if (!isset($data['is_active'])) {
            $data['is_active'] = 1;
        }
        if (empty($data['coupon_type'])) {
            $data['coupon_type'] = 2;
        }
        if (empty($data['uses_per_coupon'])) {
            $data['uses_per_coupon'] = 1;
        }
        if (empty($data['uses_per_customer'])) {
            $data['uses_per_customer'] = 1;
        }
        if (empty($data['is_rss'])) {
            $data['is_rss'] = 0;
        }
        if (isset($data['simple_action']) && $data['simple_action'] == 'by_percent' && isset($data['discount_amount'])) {
            $data['discount_amount'] = min(100, $data['discount_amount']);
        }
        if (isset($data['rule']['conditions'])) {
            $data['conditions'] = $data['rule']['conditions'];
        }
        if (isset($data['rule']['actions'])) {
            $data['actions'] = $data['rule']['actions'];
        }
        if (empty($data['from_date'])) {
            $data['from_date'] = date('Y-m-d');
        }
        if (empty($data['to_date'])) {
            $data['to_date'] = date('Y-m-d', strtotime('+1day'));
        }
        unset($data['rule']);

        /**
         * MPLUGIN-3803: Set sale rule from JMango360
         */
        $data['japi'] = 1;

        $model->loadPost($data);

        $useAutoGeneration = (int)!empty($data['use_auto_generation']);
        $model->setUseAutoGeneration($useAutoGeneration);

        $model->save();

        $output = $model->getData();
        unset($output['discount_qty']);

        return $output;
    }

    /**
     * Convert dates in array from localized to internal format
     *
     * @param array $array
     * @param array $dateFields
     * @return  array
     */
    protected function _filterDates($array, $dateFields)
    {
        if (empty($dateFields)) {
            return $array;
        }
        $filterInput = new Zend_Filter_LocalizedToNormalized(array(
            'date_format' => Mage::app()->getLocale()->getDateFormat(Mage_Core_Model_Locale::FORMAT_TYPE_SHORT)
        ));
        $filterInternal = new Zend_Filter_NormalizedToLocalized(array(
            'date_format' => Varien_Date::DATE_INTERNAL_FORMAT
        ));

        foreach ($dateFields as $dateField) {
            if (array_key_exists($dateField, $array) && !empty($dateField)) {
                $array[$dateField] = $filterInput->filter($array[$dateField]);
                $array[$dateField] = $filterInternal->filter($array[$dateField]);
            }
        }
        return $array;
    }

    /**
     * @return Jmango360_Japi_Model_Request
     */
    protected function _getRequest()
    {
        return Mage::helper('japi')->getRequest();
    }

    /**
     * @return Jmango360_Japi_Model_Response
     */
    protected function _getResponse()
    {
        return Mage::helper('japi')->getResponse();
    }

    /**
     * @return Jmango360_Japi_Model_Server
     */
    protected function _getServer()
    {
        return Mage::helper('japi')->getServer();
    }
}
