<?php
/**
 * Copyright 2020 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Rest_Hook
 */
class Jmango360_Japi_Model_Rest_Hook extends Mage_Core_Model_Abstract
{
    public function dispatch()
    {
        $action = $this->_getRequest()->getAction();
        $operation = $this->_getRequest()->getOperation();

        switch ($action . $operation) {
            case 'getHooks' . Jmango360_Japi_Model_Request::OPERATION_RETRIEVE:
                $data = $this->_getHooks();
                $this->_getResponse()->render($data);
                $this->_getResponse()->setHttpResponseCode(Jmango360_Japi_Model_Server::HTTP_OK);
                break;
            case 'create' . Jmango360_Japi_Model_Request::OPERATION_CREATE:
                $data = $this->_createHook();
                $this->_getResponse()->render($data);
                $this->_getResponse()->setHttpResponseCode(Jmango360_Japi_Model_Server::HTTP_OK);
                break;
            case 'delete' . Jmango360_Japi_Model_Request::OPERATION_DELETE:
                $data = $this->_deleteHook();
                $this->_getResponse()->render($data);
                $this->_getResponse()->setHttpResponseCode(Jmango360_Japi_Model_Server::HTTP_OK);
                break;
            default:
                throw new Jmango360_Japi_Exception(
                    Mage::helper('japi')->__('Resource method not implemented'),
                    Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR
                );
                break;
        }
    }

    protected function _getHooks()
    {
        /** @var Jmango360_Japi_Helper_Hook $helper */
        $helper = Mage::helper('japi/hook');
        return array('hooks' => $helper->getHooks());
    }

    protected function _createHook()
    {
        /** @var Jmango360_Japi_Helper_Hook $helper */
        $helper = Mage::helper('japi/hook');

        $value = $this->_getRequest()->getBodyParams();
        if (!isset($value['is_active'])) {
            $value['is_active'] = true;
        }

        $value['hid'] = uniqid('h');

        $hooks = $helper->getHooks();

        $found = false;
        foreach ($hooks as $hook) {
            if (isset($hook['scope']) && isset($value['scope']) && $hook['scope'] == $value['scope'] &&
                isset($hook['destination']) && isset($value['destination']) && $hook['destination'] == $value['destination']
            ) {
                $found = true;
                $value = $hook;
            }
        }

        if (!$found) {
            $hooks[] = $value;
            $helper->saveHooks($hooks);
        }

        return $value;
    }

    protected function _deleteHook()
    {
        $hookId = $this->_getRequest()->getParam('hook_id');
        if (!$hookId) {
            throw new Jmango360_Japi_Exception('Hook ID is missing', Jmango360_Japi_Model_Request::HTTP_BAD_REQUEST);
        }

        /** @var Jmango360_Japi_Helper_Hook $helper */
        $helper = Mage::helper('japi/hook');
        $hooks = $helper->getHooks();
        $found = false;
        foreach ($hooks as $index => $hook) {
            if ($hook['hid'] == $hookId) {
                $found = $hook;
                unset($hooks[$index]);
                break;
            }
        }

        if ($found) {
            $helper->saveHooks($hooks);

            return $found;
        } else {
            throw new Jmango360_Japi_Exception('Hook not found', Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR);
        }
    }

    /**
     * @return Jmango360_Japi_Model_Request
     */
    protected function _getRequest()
    {
        return $this->_getServer()->getRequest();
    }

    /**
     * @return Jmango360_Japi_Model_Response
     */
    protected function _getResponse()
    {
        return $this->_getServer()->getResponse();
    }

    /**
     * @return Jmango360_Japi_Model_Server
     */
    protected function _getServer()
    {
        return Mage::helper('japi')->getServer();
    }
}
