<?php

class Jmango360_Japi_Model_Rest_Product_Crosssell extends Jmango360_Japi_Model_Rest_Product
{
    const LIMIT = 99;

    /**
     * Get cross-sell product
     *
     * @return mixed
     * @throws
     */
    public function getList()
    {
        $id = $this->_getRequest()->getParam('product_id', null);
        if ($id) {
            return $this->_getListByProduct();
        } else {
            return $this->_getListByCart();
        }
    }

    /**
     * Get cross-sell products by a product
     *
     * @return mixed
     * @throws Jmango360_Japi_Exception
     * @throws Mage_Core_Exception
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function _getListByProduct()
    {
        $product = $this->_initProduct();

        if (Mage::helper('core')->isModuleEnabled('Enterprise_TargetRule')) {
            Mage::register('product', $product, true);
            $productBlock = Mage::helper('japi')->getBlock(
                'enterprise_targetrule/checkout_cart_crosssell',
                'crosssellProducts',
                array('product' => $product, 'store' => Mage::app()->getStore()));

            /* @var $helper Jmango360_Japi_Helper_Product */
            $helper = Mage::helper('japi/product');
            $productCollection = $this->_getEnterpriseCrosssellProductCollection($productBlock);

            $data['products'] = $helper->convertProductCollectionToApiResponseV2($productCollection);

            return $data;
        } else {
            $collection = $product->getCrossSellProductCollection()
                ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
                ->addAttributeToSelect(array('image', 'small_image', 'thumbnail'))
                ->setPositionOrder()
                ->addStoreFilter();

            Mage::helper('japi/product')->applyHideOnAppFilter($collection);
            Mage::getSingleton('catalog/product_visibility')->addVisibleInCatalogFilterToCollection($collection);

            /**
             * MPLUGIN-3045: Prevent collection load before append attributes for mobile
             */
            //$collection->load();

            /* @var $helper Jmango360_Japi_Helper_Product */
            $helper = Mage::helper('japi/product');

            $data['products'] = $helper->convertProductCollectionToApiResponseV2($collection);

            return $data;
        }
    }

    /**
     * Get cross-sell products by products in cart
     *
     * @return mixed
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function _getListByCart()
    {
        if (Mage::helper('core')->isModuleEnabled('Enterprise_TargetRule')) {
            $productBlock = Mage::helper('japi')->getBlock(
                'enterprise_targetrule/checkout_cart_crosssell',
                'crosssellProducts',
                array('store' => Mage::app()->getStore()));

            /* @var $helper Jmango360_Japi_Helper_Product */
            $helper = Mage::helper('japi/product');
            $productCollection = $this->_getEnterpriseCrosssellProductCollection($productBlock);

            $data['products'] = $helper->convertProductCollectionToApiResponseV2($productCollection);

            return $data;
        } else {
            $ninProductIds = $this->_getCartProductIds();
            if ($ninProductIds) {
                $filterProductIds = array_merge($this->_getCartProductIds(), $this->_getCartProductIdsRel());
                $collection = $this->_getCollection()
                    ->addProductFilter($filterProductIds)
                    ->addExcludeProductFilter($ninProductIds)
                    ->setPageSize(self::LIMIT)
                    ->setGroupBy()
                    ->setPositionOrder();

                /* @var $helper Jmango360_Japi_Helper_Product */
                $helper = Mage::helper('japi/product');

                $data['products'] = $helper->convertProductCollectionToApiResponseV2($collection);
            } else {
                $data['products'] = array();
            }

            return $data;
        }
    }

    /**
     * Get cross-sell products in EE version
     *
     * @param Enterprise_TargetRule_Block_Checkout_Cart_Crosssell $productBlock
     * @return array|Mage_Catalog_Model_Resource_Product_Collection
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function _getEnterpriseCrosssellProductCollection(Enterprise_TargetRule_Block_Checkout_Cart_Crosssell $productBlock)
    {
        $items = $productBlock->getItemCollection();

        $productIds = array();
        if (!isset($items) || empty($items)) {
            return $productIds;
        }
        foreach ($items as $item) {
            $productIds[] = $item->getEntityId();
        }

        /* @var $helper Jmango360_Japi_Helper_Product */
        $helper = Mage::helper('japi/product');

        /* @var $productCollection Mage_Catalog_Model_Resource_Product_Collection */
        $productCollection = Mage::getResourceModel('catalog/product_collection')
            ->addIdFilter($productIds)
            ->setStoreId(Mage::app()->getStore()->getId())
            ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
            ->addMinimalPrice()
            ->addFinalPrice()
            ->addTaxPercents()
            ->addUrlRewrite();

        Mage::getSingleton('catalog/product_status')->addVisibleFilterToCollection($productCollection);
        Mage::getSingleton('catalog/product_visibility')->addVisibleInCatalogFilterToCollection($productCollection);
        Mage::helper('japi/product')->applyHideOnAppFilter($productCollection);

        /* @var $resource Mage_Core_Model_Resource */
        $resource = Mage::getSingleton('core/resource');
        $productCollection->getSelect()->order(sprintf("find_in_set(e.entity_id, '%s')", implode(',', $productIds)));
        $productCollection->getSelect()
            ->join(
                array('p' => $resource->getTableName('catalog/product')),
                sprintf(
                    'e.entity_id = p.entity_id AND p.type_id IN (%s)',
                    join(',', array('"simple"', '"configurable"', '"grouped"', '"bundle"'))
                ),
                null
            );

        if (!$productCollection->getSize()) {
            $data['message'] = Mage::helper('japi')->__('No products found.');
        }

        $productCollection->clear();
        return $productCollection;
    }

    /**
     * Get ids of products that are in cart
     *
     * @return array
     */
    protected function _getCartProductIds()
    {
        $ids = array();
        foreach ($this->getQuote()->getAllItems() as $item) {
            if ($product = $item->getProduct()) {
                $ids[] = $product->getId();
            }
        }

        return $ids;
    }

    /**
     * Retrieve Array of product ids which have special relation with products in Cart
     * For example simple product as part of Grouped product
     *
     * @return array
     */
    protected function _getCartProductIdsRel()
    {
        $productIds = array();
        foreach ($this->getQuote()->getAllItems() as $quoteItem) {
            $productTypeOpt = $quoteItem->getOptionByCode('product_type');
            if ($productTypeOpt instanceof Mage_Sales_Model_Quote_Item_Option
                && $productTypeOpt->getValue() == Mage_Catalog_Model_Product_Type_Grouped::TYPE_CODE
                && $productTypeOpt->getProductId()
            ) {
                $productIds[] = $productTypeOpt->getProductId();
            }
        }

        return $productIds;
    }

    /**
     * Get last product ID that was added to cart and remove this information from session
     *
     * @return int
     */
    protected function _getLastAddedProductId()
    {
        return Mage::getSingleton('checkout/session')->getLastAddedProductId(true);
    }

    /**
     * Get crosssell products collection
     *
     * @return Mage_Catalog_Model_Resource_Eav_Mysql4_Product_Link_Product_Collection
     * @throws
     */
    protected function _getCollection()
    {
        $collection = Mage::getModel('catalog/product_link')->useCrossSellLinks()
            ->getProductCollection()
            ->setStoreId(Mage::app()->getStore()->getId())
            ->addStoreFilter()
            ->setPageSize(self::LIMIT);
        $this->_addProductAttributesAndPrices($collection);

        Mage::helper('japi/product')->applyHideOnAppFilter($collection);
        Mage::getSingleton('catalog/product_status')->addSaleableFilterToCollection($collection);
        Mage::getSingleton('catalog/product_visibility')->addVisibleInCatalogFilterToCollection($collection);
        Mage::getSingleton('cataloginventory/stock')->addInStockFilterToCollection($collection);

        return $collection;
    }

    /**
     * Add all attributes and apply pricing logic to products collection
     * to get correct values in different products lists.
     * E.g. crosssells, upsells, new products, recently viewed
     *
     * @param Mage_Catalog_Model_Resource_Product_Collection $collection
     * @return Mage_Catalog_Model_Resource_Product_Collection
     */
    protected function _addProductAttributesAndPrices(Mage_Catalog_Model_Resource_Product_Collection $collection)
    {
        return $collection
            ->addMinimalPrice()
            ->addFinalPrice()
            ->addTaxPercents()
            ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
            ->addAttributeToSelect(array('image', 'small_image', 'thumbnail'))
            ->addUrlRewrite();
    }

    /**
     * Get quote instance
     *
     * @return Mage_Sales_Model_Quote
     */
    protected function getQuote()
    {
        return Mage::getSingleton('checkout/session')->getQuote();
    }
}