<?php

class Jmango360_Japi_Model_Rest_Product_Related extends Jmango360_Japi_Model_Rest_Product
{
    public function getList()
    {
        $product = $this->_initProduct();

        if (Mage::helper('core')->isModuleEnabled('Enterprise_TargetRule')) {
            Mage::register('product', $product, true);
            $productBlock = Mage::helper('japi')->getBlock(
                'enterprise_targetrule/catalog_product_list_related',
                'relatedProducts',
                array('product' => $product, 'store' => Mage::app()->getStore()));
            $items = $productBlock->getItemCollection();

            $productIds = array();
            if (!isset($items) || empty($items)) {
                return $productIds;
            }
            foreach ($items as $item) {
                $productIds[] = $item->getEntityId();
            }

            /* @var $helper Jmango360_Japi_Helper_Product */
            $helper = Mage::helper('japi/product');

            /* @var $productCollection Mage_Catalog_Model_Resource_Product_Collection */
            $productCollection = Mage::getResourceModel('catalog/product_collection')
                ->addIdFilter($productIds)
                ->setStoreId(Mage::app()->getStore()->getId())
                ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
                ->addMinimalPrice()
                ->addFinalPrice()
                ->addTaxPercents()
                ->addUrlRewrite();

            Mage::getSingleton('catalog/product_status')->addVisibleFilterToCollection($productCollection);
            Mage::getSingleton('catalog/product_visibility')->addVisibleInCatalogFilterToCollection($productCollection);
            Mage::helper('japi/product')->applyHideOnAppFilter($productCollection);

            /* @var $resource Mage_Core_Model_Resource */
            $resource = Mage::getSingleton('core/resource');
            $productCollection->getSelect()->order(sprintf("find_in_set(e.entity_id, '%s')", implode(',', $productIds)));
            $productCollection->getSelect()
                ->join(
                    array('p' => $resource->getTableName('catalog/product')),
                    sprintf(
                        'e.entity_id = p.entity_id AND p.type_id IN (%s)',
                        join(',', array('"simple"', '"configurable"', '"grouped"', '"bundle"'))
                    ),
                    null
                );

            if (!$productCollection->getSize()) {
                $data['message'] = Mage::helper('japi')->__('No products found.');
            }

            $productCollection->clear();

            $data['products'] = $helper->convertProductCollectionToApiResponseV2($productCollection);

            return $data;
        } else {
            $collection = $product->getRelatedProductCollection()
                ->addAttributeToSelect('required_options')
                ->setPositionOrder()
                ->addStoreFilter();

            if (Mage::helper('core')->isModuleEnabled('GGMGastro_Catalog')) {
                $collection->getSelect()->order(new Zend_Db_Expr('RAND()'));
            }
            if (Mage::helper('catalog')->isModuleEnabled('Mage_Checkout')) {
                Mage::getResourceSingleton('checkout/cart')->addExcludeProductFilter(
                    $collection,
                    Mage::getSingleton('checkout/session')->getQuoteId()
                );
                $this->_addProductAttributesAndPrices($collection);
            }

            Mage::helper('japi/product')->applyHideOnAppFilter($collection);
            Mage::getSingleton('catalog/product_visibility')->addVisibleInCatalogFilterToCollection($collection);

            /**
             * MPLUGIN-3045: Prevent collection load before append attributes for mobile
             */
            //$collection->load();

            /* @var $helper Jmango360_Japi_Helper_Product */
            $helper = Mage::helper('japi/product');

            $data['products'] = $helper->convertProductCollectionToApiResponseV2($collection);

            return $data;
        }
    }

    /**
     * Add all attributes and apply pricing logic to products collection
     * to get correct values in different products lists.
     * E.g. crosssells, upsells, new products, recently viewed
     *
     * @param Mage_Catalog_Model_Resource_Product_Collection $collection
     * @return Mage_Catalog_Model_Resource_Product_Collection
     */
    protected function _addProductAttributesAndPrices(Mage_Catalog_Model_Resource_Product_Collection $collection)
    {
        return $collection
            ->addMinimalPrice()
            ->addFinalPrice()
            ->addTaxPercents()
            ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
            ->addAttributeToSelect(array('image', 'small_image', 'thumbnail'))
            ->addUrlRewrite();
    }
}
