<?php

class Jmango360_Japi_Model_Rest_Product_Search extends Jmango360_Japi_Model_Rest_Catalog_Search_Products
{
    /**
     * Get search result
     *
     * @throws
     */
    public function getList()
    {
        /* @var $helper Jmango360_Japi_Helper_Product */
        $helper = Mage::helper('japi/product');

        $q = Mage::helper('japi')->getRequest()->getParam('q', false);
        $attributeCode = Mage::helper('japi')->getRequest()->getParam('code', false);
        $attributeValue = Mage::helper('japi')->getRequest()->getParam('value', false);

        if ($attributeCode && $attributeValue) {
            /* @var $productCollection Mage_Catalog_Model_Resource_Product_Collection */
            $productCollection = Mage::getResourceModel('catalog/product_collection')
                ->setStoreId($this->getStoreId())
                ->addAttributeToSelect(Mage::getSingleton('catalog/config')->getProductAttributes())
                ->addMinimalPrice()
                ->addFinalPrice()
                ->addTaxPercents()
                ->addUrlRewrite();

            Mage::getSingleton('catalog/product_status')->addVisibleFilterToCollection($productCollection);
            Mage::getSingleton('catalog/product_visibility')->addVisibleInCatalogFilterToCollection($productCollection);

            $helper->applyHideOnAppFilter($productCollection);
            $helper->addAttributeToSearch($productCollection, $attributeCode, $attributeValue);

            return $this->prepareProductResponse($productCollection);
        } else if ($q) {
            /* @var $searchHelper Mage_CatalogSearch_Helper_Data */
            $searchHelper = Mage::helper('catalogsearch');
            /* @var $query Mage_CatalogSearch_Model_Query */
            $query = $searchHelper->getQuery();

            /**
             * Support Emico_Tweakwise
             */
            if ($helper->isModuleEnabled('Emico_Tweakwise')) {
                return $helper->getProductCollectionFromEmicoTweakwise(true);
            }

            /**
             * Support Klevu_Search
             */
            if ($helper->isModuleEnabled('Klevu_Search')) {
                Mage::app()->getStore()->setConfig('klevu_search/searchlanding/landenabled', 1);
                if (Mage::helper("klevu_search/config")->isLandingEnabled() == 1 &&
                    Mage::helper("klevu_search/config")->isExtensionConfigured()) {
                    /* @var $obj Klevu_Search_Model_Observer */
                    $obj = Mage::getSingleton('klevu_search/observer');
                    if (method_exists($obj, 'applyLandingPageModelRewrites')) {
                        $obj->applyLandingPageModelRewrites(new Varien_Event_Observer());
                    }

                    return $helper->getProductCollectionFromKlevuSearch();
                }
            }

            /**
             * Support SolrBridge_Solrsearch
             */
            if ($helper->isModuleEnabled('SolrBridge_Solrsearch')) {
                return $helper->getProductCollectionFromSolrBridgeSolrsearch();
            }

            $block = $this->_getSearchLayerBlock();
            $productCollection = $block->getLayer()->getProductCollection();

            /**
             * Support Algolia_Algoliasearch
             */
            if ($helper->isModuleEnabled('Algolia_Algoliasearch')) {
                try {
                    /** @var Algolia_Algoliasearch_Helper_Config $config */
                    $config = Mage::helper('algoliasearch/config');
                    $storeId = Mage::app()->getStore()->getId();
                    if ($config->getApplicationID() && $config->getAPIKey() && $config->isEnabledFrontEnd($storeId)) {
                        /** @var Algolia_Algoliasearch_Helper_Data $algoliaSearch */
                        $algoliaSearch = Mage::helper('algoliasearch');
                        $data = $algoliaSearch->getSearchResult($query->getQueryText(), $storeId);
                        if (count($data)) {
                            $productCollection->addIdFilter(array_keys($data));
                        }
                    }
                } catch (Exception $e) {
                    // ignore
                }
            }

            if ($searchHelper->isMinQueryLength()) {
                $query->setId(0)
                    ->setIsActive(1)
                    ->setIsProcessed(1);
            } else {
                if ($query->getId()) {
                    $query->setPopularity($query->getPopularity() + 1);
                } else {
                    $query->setPopularity(1);
                }

                $query->prepare();
            }

            $helper->applyHideOnAppFilter($productCollection);

            $data = $this->prepareProductResponse($productCollection, $block);

            if (!$searchHelper->isMinQueryLength()) {
                $query->save();
            }

            return $data;
        } else {
            throw new Jmango360_Japi_Exception(
                Mage::helper('japi')->__('Query cannot be empty.'),
                Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR
            );
        }
    }

    /**
     * @param Mage_Catalog_Model_Resource_Product_Collection $productCollection
     * @param Jmango360_Japi_Block_Catalogsearch_Layer $block
     * @return array
     */
    protected function prepareProductResponse($productCollection, $block = null)
    {
        $data = array();
        /* @var $helper Jmango360_Japi_Helper_Product */
        $helper = Mage::helper('japi/product');
        /* @var $searchHelper Mage_CatalogSearch_Helper_Data */
        $searchHelper = Mage::helper('catalogsearch');
        /* @var $resource Mage_Core_Model_Resource */
        $resource = Mage::getSingleton('core/resource');

        $productCollection->getSelect()->join(
            array('p' => $resource->getTableName('catalog/product')),
            sprintf(
                'e.entity_id = p.entity_id AND p.type_id IN (%s)',
                join(',', array('"simple"', '"configurable"', '"grouped"', '"bundle"'))
            ),
            null
        );

        if (!$productCollection->getSize()) {
            $data['message'] = $searchHelper->__('Your search returns no results.');
        }

        $data['filters'] = $this->_getFilters($block);
        $helper->addPageSettings($productCollection);
        $data['toolbar_info'] = $helper->getToolbarInfo($productCollection, true);

        $productCollection->clear();
        $data['products'] = $helper->convertProductCollectionToApiResponseV2($productCollection);

        $searchHelper->checkNotes();
        $messages = $searchHelper->getNoteMessages();
        if (!empty($messages)) {
            $data['message'] .= implode("\n", (array)$messages);
        }

        return $data;
    }

    /**
     * Get fast search result
     */
    public function getSuggest()
    {
        /* @var $searchHelper Mage_CatalogSearch_Helper_Data */
        $searchHelper = Mage::helper('catalogsearch');
        /* @var $query Mage_CatalogSearch_Model_Query */
        $query = $searchHelper->getQuery();
        $query->setStoreId(Mage::app()->getStore()->getId());
        $data = array();

        if ($query->getQueryText() != '') {
            if ($searchHelper->isMinQueryLength()) {
                $query->setId(0)->setIsActive(1)->setIsProcessed(1);
            } else {
                if ($query->getId()) {
                    $query->setPopularity($query->getPopularity() + 1);
                } else {
                    $query->setPopularity(1);
                }
                $query->prepare();
            }

            $helper = Mage::helper('japi/product');
            /* @var $helper Jmango360_Japi_Helper_Product */
            $block = $this->_getSearchLayerBlock();
            /* @var $productCollection Mage_Catalog_Model_Resource_Product_Collection */
            $productCollection = $block->getLayer()->getProductCollection();
            /* @var $resource Mage_Core_Model_Resource */
            $resource = Mage::getSingleton('core/resource');

            $productCollection->getSelect()->join(
                array('p' => $resource->getTableName('catalog/product')),
                sprintf(
                    'e.entity_id = p.entity_id AND p.type_id IN (%s)',
                    join(',', array('"simple"', '"configurable"', '"grouped"', '"bundle"'))
                ),
                null
            );

            if (!$productCollection->getSize()) {
                $data['message'] = $searchHelper->__('Your search returns no results.');
            }

            $helper->addPageSettings($productCollection);
            $helper->getToolBarInfo($productCollection, true);

            $data['products'] = $helper->convertSuggestProductCollectionToApiResponse($productCollection);

            $searchHelper->checkNotes();
            $messages = $searchHelper->getNoteMessages();
            if (!empty($messages)) {
                $data['message'] .= implode("\n", (array)$messages);
            }

            if (!$searchHelper->isMinQueryLength()) {
                $query->save();
            }
        } else {
            throw new Jmango360_Japi_Exception(
                $searchHelper->__('Query cannot be empty.'),
                Jmango360_Japi_Model_Request::HTTP_INTERNAL_ERROR
            );
        }

        return $data;
    }
}