<?php
/**
 * Copyright 2018 JMango360
 */

require_once 'JapiOrchardConnecter.php';
require_once 'JapiOrchardConnecterLogger.php';

/**
 * Class Jmango360_Japi_Adminhtml_Japi_PushmessageController
 */
class Jmango360_Japi_Adminhtml_Japi_PushmessageController extends Mage_Adminhtml_Controller_Action
{
    protected function _isAllowed()
    {
        return Mage::getSingleton('admin/session')->isAllowed('japi/pushmessage');
    }

    public function indexAction()
    {
        Mage::getSingleton('adminhtml/session')->setData('orchard_url', Mage::app()->getRequest()->getParam('base_url'));
        Mage::getSingleton('adminhtml/session')->setData('orchard_ticket', Mage::app()->getRequest()->getParam('ticket'));
        Mage::getSingleton('adminhtml/session')->setData('orchard_appKey', Mage::app()->getRequest()->getParam('appKey'));
        Mage::getSingleton('adminhtml/session')->setData('orchard_storeId', Mage::app()->getRequest()->getParam('store_id'));

        $this->loadLayout()->renderLayout();

        //Clear "X-Frame-Options" header which prevent this page display in a iframe
        $this->getResponse()->clearHeader('X-Frame-Options');
    }

    /**
     * Load recipient data action
     */
    public function loadAction()
    {
        $output = array();
        $id = $this->getRequest()->getParam('id');
        switch ($id) {
            case 'USERS':
                $output = $this->getUsers(array(
                    'type' => $id,
                    'icon' => 'fas fa-user color-3793E6',
                    'limit' => $this->getRequest()->getParam('limit', 10),
                    'page' => $this->getRequest()->getParam('page', 1),
                    'query' => $this->getRequest()->getParam('query')
                ));
                break;
            case 'MAGENTO_USER_GROUP':
                $output = $this->getMagentoGroups(array(
                    'type' => $id,
                    'icon' => 'fab fa-magento color-FF4800'
                ));
                break;
            default:
                $output['error'] = 1;
                $output['message'] = $this->__('Data not supported');
        }

        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($output));
    }

    /**
     * Send push message action
     */
    public function sendAction()
    {
        if ($this->getRequest()->isPost()) {
            $output = array();
            $data = array();

            $deepLink = $this->getRequest()->getParam('deep_link');
            if ($deepLink == 1 || $deepLink == 2) {//category or module
                $data['moduleType'] = $this->getRequest()->getParam('moduleType');
                $data['moduleName'] = $this->getRequest()->getParam('moduleName');
                $data['moduleId'] = $this->getRequest()->getParam('moduleId');
            } elseif ($deepLink == 3) {//product
                $data['productUrl'] = $this->getRequest()->getParam('productUrl');
                $data['productId'] = $this->getRequest()->getParam('productId');
            }

            $data['message'] = $this->getRequest()->getParam('body', '');

            $recipient = $this->getRequest()->getParam('recipient');
            if (is_array($recipient)) {
                foreach ($recipient as $type => $values) {
                    switch ($type) {
                        case 'MAGENTO_USER_GROUP':
                            $data['userGroups'] = array();
                            $values = is_array($values) ? $values : array($values);
                            foreach ($values as $groupId) {
                                /** @var Mage_Customer_Model_Group $groupModel */
                                $groupModel = Mage::getModel('customer/group')->load($groupId);
                                $data['userGroups'][] = array(
                                    'id' => $groupId,
                                    'name' => $groupModel->getId() ? $groupModel->getCode() : '',
                                    'users' => $this->convertUserIdToRequest($this->getJapiCustomerFromGroup($groupId))
                                );
                            }
                            break;
                        case 'USERS':
                            $data['users'] = array();
                            foreach ($values as $userId) {
                                $data['users'][] = array('id' => $userId);
                            }
                            break;
                    }
                }
            }

            try {
                $connector = $this->_getOrchardConnector();
                $output = $connector->pushMessage($data);
            } catch (Exception $e) {
                $output['error'] = 1;
                $output['message'] = $e->getMessage();
            }

            $this->getResponse()->setHeader('Content-type', 'application/json');
            $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($output));
        } else {
            $this->getResponse()->setRedirect(Mage::helper('adminhtml')->getUrl('*/*/index'));
        }
    }

    /**
     * Convert array of Ids to format accept by Orchard
     *
     * @param $ids
     * @return array
     */
    protected function convertUserIdToRequest($ids)
    {
        $output = array();

        foreach ($ids as $id) {
            $output[] = array('id' => $id);
        }

        return $output;
    }

    /**
     * Get all customer IDs from groups
     *
     * @param $ids
     * @return array
     */
    protected function getJapiCustomerFromGroup($ids)
    {
        if (!is_array($ids)) $ids = array($ids);

        /** @var Mage_Customer_Model_Resource_Customer_Collection $collection */
        $collection = Mage::getSingleton('customer/customer')->getCollection();
        $collection->addFieldToFilter('group_id', array('in' => $ids));
        $collection->addAttributeToFilter('japi', array('eq' => 1));

        return $collection->getAllIds();
    }

    /**
     * Get data for Deep link function
     */
    public function productAction()
    {
        try {
            $output = $this->getProduct(array(
                'limit' => $this->getRequest()->getParam('limit', 10),
                'page' => $this->getRequest()->getParam('page', 1),
                'query' => $this->getRequest()->getParam('query')
            ));
        } catch (Exception $e) {
            $output['error'] = 1;
            $output['message'] = $e->getMessage();
        }

        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($output));
    }

    /**
     * Get connection object with Orchard
     *
     * @return JapiOrchardConnecter
     * @throws
     */
    protected function _getOrchardConnector()
    {
        $baseUrl = Mage::getSingleton('adminhtml/session')->getData('orchard_url');
        $ticket = Mage::getSingleton('adminhtml/session')->getData('orchard_ticket');
        $appKey = Mage::getSingleton('adminhtml/session')->getData('orchard_appKey');

        $connector = new JapiOrchardConnecter($baseUrl, $ticket, $appKey);
        $connector->setLogger(new JapiOrchardConnecterLogger());

        return $connector;
    }

    /**
     * Get module data from Orchard
     */
    public function moduleAction()
    {
        try {
            $output = array();
            $connecter = $this->_getOrchardConnector();
            $result = $connecter->getCategoryTreeAndModules();
            $output = empty($result['modules']) ? array() : $result['modules'];
        } catch (Exception $e) {
            $output['error'] = 1;
            $output['message'] = $e->getMessage();
        }

        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($output));
    }

    /**
     * Get category data
     *
     * @throws
     */
    public function categoryAction()
    {
        try {
            $connecter = $this->_getOrchardConnector();
            $result = $connecter->getCategoryTreeAndModules();
            $output = empty($result['categories']) ? array() : $result['categories'];
        } catch (Exception $e) {
            $output = array();
        }

        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($output));
    }


    /**
     * Get product by name
     *
     * @param array $params
     * @return array
     * @throws
     */
    protected function getProduct($params = array())
    {
        if (empty($params['query'])) {
            return array();
        }

        $storeId = Mage::getSingleton('adminhtml/session')->getData('orchard_storeId');

        /** @var Mage_Catalog_Model_Resource_Product_Collection $collection */
        $collection = Mage::getSingleton('catalog/product')->getCollection();
        $collection->setStore($storeId);
        $collection->addAttributeToFilter('name', array('like' => sprintf('%%%s%%', $params['query'])));
        $collection->addUrlRewrite();
        Mage::getSingleton('catalog/product_visibility')->addVisibleInSiteFilterToCollection($collection);
        $collection->setPage($params['page'], $params['limit']);

        $output = $this->_prepareData($params);
        $output['total'] = $collection->getSize();

        foreach ($collection as $item) {
            /** @var $item Mage_Catalog_Model_Product */
            $output['items'][] = array(
                'id' => $item->getId(),
                'name' => $item->getName(),
                'url' => $item->getProductUrl()
            );
        }

        return $output;
    }

    /**
     * Prepare response data
     *
     * @param $params
     * @return array
     */
    protected function _prepareData($params = array())
    {
        return array(
            'type' => isset($params['type']) ? $params['type'] : null,
            'icon' => isset($params['icon']) ? $params['icon'] : null,
            'limit' => isset($params['limit']) ? (int)$params['limit'] : 999,
            'page' => isset($params['page']) ? (int)$params['page'] : 1,
            'query' => isset($params['query']) ? $params['query'] : null
        );
    }

    /**
     * Load users from Magento with JAPI flag check
     *
     * @param $params
     * @return array
     */
    protected function getUsers($params = array())
    {
        /** @var Mage_Customer_Model_Resource_Customer_Collection $collection */
        $collection = Mage::getSingleton('customer/customer')->getCollection();
        $collection->addAttributeToFilter('japi', 1);
        if (!empty($params['query'])) {
            $collection->addFieldToFilter('email', array('like' => sprintf('%%%s%%', $params['query'])));
        }
        $collection->setPage($params['page'], $params['limit']);

        $output = $this->_prepareData($params);
        $output['total'] = $collection->getSize();

        foreach ($collection as $item) {
            $output['options'][] = array(
                'id' => 'u' . $item->getId(),
                'oid' => $item->getId(),
                'label' => $item->getEmail()
            );
        }

        return $output;
    }

    /**
     * Load Magento groups
     *
     * @param $params
     * @return array
     */
    protected function getMagentoGroups($params = array())
    {
        $output = $this->_prepareData($params);

        $collection = Mage::getSingleton('customer/group')->getCollection();
        foreach ($collection as $item) {
            /** @var $item Mage_Customer_Model_Group */
            if ($item->getId() == 0) continue;

            /** @var Mage_Customer_Model_Resource_Customer_Collection $collection */
            $collection = Mage::getSingleton('customer/customer')->getCollection();
            $collection->addFieldToFilter('group_id', array('eq' => $item->getId()));
            $collection->addAttributeToFilter('japi', 1);
            if (!$collection->getSize()) continue;

            $output['options'][] = array(
                'id' => 'g' . $item->getId(),
                'oid' => $item->getId(),
                'label' => $item->getCode(),
                'count' => $collection->getSize()
            );
        }

        return $output;
    }
}
