<?php
/**
 * Copyright 2015 JMango360
 */

if (Mage::helper('core')->isModuleEnabled('Flagbit_Checkout')) {
    include_once 'CheckoutController_Flagbit.php';
} elseif (Mage::helper('core')->isModuleEnabled('MadeByMouses_Olifant')) {
    include_once 'CheckoutController_Olifant.php';
} else {
    include_once 'CheckoutController_Jmango360.php';
}

class Jmango360_Japi_CheckoutController extends Jmango360_Japi_Checkout_Abstract
{
    /**
     * Make sure customer is valid, if logged in
     * By default will add error messages and redirect to customer edit form
     *
     * @param bool $redirect - stop dispatch and redirect?
     * @param bool $addErrors - add error messages?
     * @return bool
     */
    protected function _preDispatchValidateCustomer($redirect = true, $addErrors = true)
    {
        $customer = Mage::getSingleton('customer/session')->getCustomer();
        if ($customer && $customer->getId()) {
            $validationResult = $customer->validate();
            if ((true !== $validationResult) && is_array($validationResult)) {
                if ($this->getRequest()->getActionName() == 'new') {
                    Mage::getSingleton('customer/session')->setNewCheckoutReferer(true);
                } else {
                    Mage::getSingleton('customer/session')->setCheckoutReferer(true);
                }

                if ($addErrors) {
                    foreach ($validationResult as $error) {
                        Mage::getSingleton('customer/session')->addError($error);
                    }
                }

                if ($redirect) {
                    $this->_redirect('japi/customer/edit', array('_secure' => true));
                    $this->setFlag('', self::FLAG_NO_DISPATCH, true);
                }

                return false;
            }
        }

        return true;
    }

    /**
     * Send Ajax redirect response
     * @param $message
     * @return Mage_Checkout_OnepageController
     */
    protected function _ajaxRedirectResponse($message = null)
    {
        $this->getResponse()
            ->setHeader('Content-Type', 'application/json')
            ->setBody(Mage::helper('core')->jsonEncode(array(
                'error' => true,
                'message' => $message ? $message : $this->__('Session exprired.')
            )));
        return $this;
    }

    /**
     * Validate ajax request and redirect on failure
     */
    protected function _expireAjax()
    {
        $quote = $this->getOnepage()->getQuote();
        if (!$quote->hasItems()) {
            $this->_ajaxRedirectResponse($this->__('Cart is empty.'));
            return true;
        } else if ($quote->getHasError()) {
            foreach ($quote->getMessages() as $message) {
                $this->_ajaxRedirectResponse($message->getCode());
            }
            return true;
        } else if ($quote->getIsMultiShipping()) {
            $this->_ajaxRedirectResponse();
            return true;
        }

        return false;
    }

    /**
     * @return Mage_Checkout_Model_Session
     */
    protected function _getSesstion()
    {
        return Mage::getSingleton('checkout/session');
    }

    /**
     * Check can page show for unregistered users
     *
     * @return boolean
     */
    protected function _allowGuestCheckout()
    {
        return Mage::getSingleton('customer/session')->isLoggedIn()
            || Mage::helper('checkout')->isAllowedGuestCheckout($this->getOnepage()->getQuote());
    }

    /**
     * Redirect to native web checkout url
     */
    public function redirectAction()
    {
        /* @var $helper Jmango360_Japi_Helper_Data */
        $helper = Mage::helper('japi');

        /**
         * MPLUGIN-1377: Reset PHPSESSID
         */
        $helper->checkValidSession();
        $helper->injectCookies();
        $helper->clearPreviousFlags();

        $checkoutUrl = $helper->getCheckoutUrl();
        if ($checkoutUrl) {
            $this->_redirectUrl($checkoutUrl);
        } else {
            $params = $this->getRequest()->getParams();
            $this->_redirect('checkout/onepage', array('_secure' => true, '_query' => $params));
        }
    }

    /**
     * New checkout UI
     */
    public function newAction()
    {
        $this->onepageAction();
    }

    /**
     * New onepage checkout page
     */
    public function onepageAction()
    {
        /* @var $server Jmango360_Japi_Model_Server */
        $server = Mage::getSingleton('japi/server');
        $server->setIsRest();

        /** @var Jmango360_Japi_Helper_Data $helper */
        $helper = Mage::helper('japi');
        $helper->injectCookies();

        /** @var $quote Mage_Sales_Model_Quote $quote */
        $quote = $this->getOnepage()->getQuote();
        $quote->collectTotals();
        if ($quote->getHasError()) {
            foreach ($quote->getMessages() as $message) {
                $this->_getSesstion()->addError($message->getCode());
            }
        }
        if (!$quote->hasItems()) {
            $this->_getSesstion()->addError(Mage::helper('checkout')->__('You have no items in your shopping cart.'));
            $quote->setHasError(true);
        }
        if (!$quote->validateMinimumAmount()) {
            $error = Mage::getStoreConfig('sales/minimum_order/error_message') ?
                Mage::getStoreConfig('sales/minimum_order/error_message') :
                Mage::helper('checkout')->__('Subtotal must exceed minimum order amount');

            $this->_getSesstion()->addError($error);
            $quote->setHasError(true);
        }
        if (!$this->_allowGuestCheckout()) {
            $this->_getSesstion()->addError(
                Mage::helper('checkout')->__('Sorry, guest checkout is not enabled. Please try again or contact store owner.')
            );
            $quote->setHasError(true);
        }
        if ($quote->getIsMultiShipping()) {
            $this->_getSesstion()->addError(
                Mage::helper('checkout')->__('Invalid checkout type.')
            );
            $quote->setHasError(true);
        }

        // Bypass merge js
        Mage::app()->getStore()->setConfig('dev/js/merge_files', 0);
        Mage::app()->getStore()->setConfig('dev/js/meanbee_footer_js_enabled', 0);
        Mage::app()->getStore()->setConfig('gtspeed/cssjs/min_js', 0);
        Mage::app()->getStore()->setConfig('gtspeed/cssjs/merge_js', 0);
        Mage::app()->getStore()->setConfig('hsmedia/mediasetting/hsmedia_enabled', 0);
        Mage::app()->getStore()->setConfig('aw_mobile2/general/is_enabled', 0);

        /**
         * MPLUGIN-2680: disable MageWorx_StoreSwitcher configs
         */
        Mage::app()->getStore()->setConfig('mageworx_geoip/mageworx_storeswitcher/enable_store_switcher', 0);
        Mage::app()->getStore()->setConfig('mageworx_geoip/mageworx_storeswitcher/enable_billing_country', 0);
        Mage::app()->getStore()->setConfig('mageworx_geoip/mageworx_storeswitcher/enable_shipping_country', 0);

        if (Mage::app()->getRequest()->getParam('JMDEBUG')) {
            Mage::app()->getStore()->setConfig('dev/debug/template_hints', 1);
            Mage::app()->getStore()->setConfig('dev/debug/template_hints_blocks', 1);
            Mage::app()->getStore()->setConfig('dev/restrict/allow_ips', null);
        }

        // Freeze cart object
        Mage::getSingleton('checkout/session')->setCartWasUpdated(false);

        /**
         * MPLUGIN-2784: Fix Onepage checkout always revert customer addresses to default addresses
         */
        if ($quote->getCustomerId()
            && !$quote->getBillingAddress()->getCustomerAddressId()
            && !$quote->getShippingAddress()->getCustomerAddressId()) {
            $this->getOnepage()->initCheckout();
        }

        /**
         * Reset checkout_method
         */
        if ($quote->getCustomerId()) {
            $quote->setCheckoutMethod(Mage_Checkout_Model_Type_Onepage::METHOD_CUSTOMER);
        } else {
            $quote->setCheckoutMethod(Mage_Checkout_Model_Type_Onepage::METHOD_GUEST);
        }

        /**
         * Clear "skip_back_to_app" if exist
         */
        /** @var Mage_Core_Model_Cookie $cookie */
        $cookie = Mage::getSingleton('core/cookie');
        if ($cookie->get('skip_back_to_app')) {
            $cookie->delete('skip_back_to_app');
        }

        /**
         * MPLUGIN-3725: Set messages from previous place order action
         */
        $messages = $this->_getSesstion()->getData('japi_messages');
        if ($messages) {
            foreach ($messages as $message) {
                switch ($message['type']) {
                    case Mage_Core_Model_Message::ERROR:
                    default:
                        $this->_getSesstion()->addError($message['message']);
                        break;
                    case Mage_Core_Model_Message::NOTICE:
                        $this->_getSesstion()->addNotice($message['message']);
                        break;
                    case Mage_Core_Model_Message::SUCCESS:
                        $this->_getSesstion()->addSuccess($message['message']);
                        break;
                    case Mage_Core_Model_Message::WARNING:
                        $this->_getSesstion()->addWarning($message['message']);
                        break;
                }
            }
            $this->_getSesstion()->unsetData('japi_messages');
        }

        $this->loadLayout();
        $this->_updateLayout();
        $this->getLayout()->getBlock('head')->setTitle($this->__('Checkout'));
        $this->_appendPopupUrls();
        $this->_initLayoutMessages(array('checkout/session'));
        $this->renderLayout();
    }

    /**
     * Update Onepage layout with custom xml in Settings
     */
    protected function _updateLayout()
    {
        /* @var $helper Jmango360_Japi_Helper_Data */
        $helper = Mage::helper('japi');
        $xml = '';

        if ($helper->isModuleEnabled('PostcodeNl_Api')) {
            if (version_compare($helper->getExtensionVersion('PostcodeNl_Api'), '1.1.0', '<')) {
                $xml .= "
<reference name=\"head\">
    <action method=\"addCss\" ifconfig=\"postcodenl/config/enabled\"><script>postcodenl/api/css/lookup.css</script></action>
    <action method=\"addJs\" ifconfig=\"postcodenl/config/enabled\"><script>postcodenl/api/lookup.js</script></action>
</reference>";
            } else {
                $xml .= "
<reference name=\"head\">
    <action method=\"addCss\" ifconfig=\"postcodenl_api/config/enabled\"><script>postcodenl/api/css/lookup.css</script></action>
    <action method=\"addJs\" ifconfig=\"postcodenl_api/config/enabled\"><script>postcodenl/api/lookup.js</script></action>
</reference>";
            }
            $xml .= "
<reference name=\"content\">
    <block type=\"postcodenl_api/jsinit\" name=\"postcodenl.jsinit\" template=\"postcodenl/api/jsinit.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Adyen_Payment')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>adyen/payment/cc.js</script></action>
    <action method=\"addJs\"><script>adyen/payment/iban.js</script></action>
    <action method=\"addJs\"><script>adyen/payment/elv.js</script></action>
    <action method=\"addCss\"><stylesheet>css/adyenstyle.css</stylesheet></action>
</reference>
<reference name=\"after_body_start\">
    <block type=\"core/text\" name=\"adyen.diners.validation\" after=\"-\">
        <action method=\"setText\">
            <text><![CDATA[<script type=\"text/javascript\">
                    Validation.creditCartTypes.set('DC', [new RegExp('^3(?:0[0-5]|[68][0-9])[0-9]{11}$'), new RegExp('^[0-9]{3}$'), true]);
                    Validation.creditCartTypes.set('CB', [new RegExp('^4[0-9]{12}([0-9]{3})?$'), new RegExp('^[0-9]{3}$'), true]);
                    Validation.creditCartTypes.set('ELO', [new RegExp(/^((((636368)|(438935)|(504175)|(451416)|(636297)|(506699))\d{0,10})|((5067)|(4576)|(4011))\d{0,12})$/), new RegExp('^[0-9]{3}$'), true]);
                    Validation.creditCartTypes.set('hipercard', [new RegExp(/^(606282\d{10}(\d{3})?)|(3841\d{15})$/), new RegExp('^[0-9]{3}$'), true]);
                    Validation.creditCartTypes.set('unionpay', [new RegExp('^62[0-5]\d{13,16}$'), new RegExp('^[0-9]{3}$'), true]);
                </script>]]></text>
        </action>
    </block>
</reference>";
        }

        if ($helper->isModuleEnabled('TIG_PostNL')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addCss\"><stylesheet>css/TIG/PostNL/deliveryoptions/default.css</stylesheet></action>
    <action method=\"addCss\"><stylesheet>css/TIG/PostNL/postcodecheck.css</stylesheet></action>
    <action method=\"addCss\"><stylesheet>css/TIG/PostNL/mijnpakket.css</stylesheet></action>
    <action method=\"addItem\"><type>skin_css</type><name>css/TIG/PostNL/deliveryoptions/ie8.css</name><params/><if>lt IE 9</if></action>
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/PostNL/ajax.js</file></action>
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/PostNL/postcodecheck.js</file></action>
    <block type=\"postnl_deliveryoptions/theme\" name=\"postnl_deliveryoptions_theme\" template=\"TIG/PostNL/delivery_options/theme.phtml\"/>
</reference>
<reference name=\"checkout.onepage.billing\">
    <block type=\"core/template\" name=\"postnl_billing_postcodecheck\" template=\"TIG/PostNL/address_validation/checkout/onepage/postcode_check.phtml\"/>
</reference>
<reference name=\"checkout.onepage.shipping\">
    <block type=\"core/template\" name=\"postnl_shipping_postcodecheck\" template=\"TIG/PostNL/address_validation/checkout/onepage/postcode_check.phtml\"/>
</reference>
<reference name=\"before_body_end\">
    <block type=\"core/template\" name=\"postnl_validation\" template=\"TIG/PostNL/address_validation/validate.phtml\"/>
    <block type=\"postnl_deliveryoptions/js\" name=\"postnl_deliveryoptions_js\" template=\"TIG/PostNL/delivery_options/js.phtml\"/>
    <block type=\"postnl_deliveryoptions/pickupNotification\" name=\"postnl_billing_pickup_notification\" template=\"TIG/PostNL/delivery_options/onepage/pickup_notification.phtml\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('Vaimo_Klarna')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>vaimo/klarna/klarna.js</script></action>
    <action method=\"addCss\"><stylesheet>css/vaimo/klarna/checkout.css</stylesheet></action>
    <block type=\"page/html_head\" name=\"klarna_header\" as=\"klarna_header\" template=\"vaimo/klarna/checkout/header.phtml\" />
</reference>
<reference name=\"content\">
    <block type=\"klarna/checkout_top\" after=\"-\" name=\"klarna_checkout_top\" as=\"klarna_checkout_top\" template=\"vaimo/klarna/checkout/top.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('AW_Deliverydate')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>varien/product.js</script></action>
    <action method=\"addCss\"><stylesheet>aw_deliverydate/css/main.css</stylesheet></action>
    <action method=\"addItem\"><type>js_css</type><name>calendar/calendar-win2k-1.css</name><params/></action>
    <action method=\"addItem\"><type>js</type><name>calendar/calendar.js</name></action>
    <action method=\"addItem\"><type>js</type><name>calendar/calendar-setup.js</name></action>
    <action method=\"addJs\"><script>jquery/jquery.1.9.1.min.js</script></action>
    <action method=\"addJs\"><script>jquery/jquery.noConflict.js</script></action>
    <action method=\"addJs\"><script>pickadate/picker.js</script></action>
    <action method=\"addJs\"><script>pickadate/picker.date.js</script></action>
    <action method=\"addItem\"><type>js_css</type><stylesheet>pickadate/theme/default.css</stylesheet></action>
    <action method=\"addItem\"><type>js_css</type><stylesheet>pickadate/theme/default.date.css</stylesheet></action>
    <block type=\"core/html_calendar\" name=\"html_calendar\" as=\"html_calendar\" template=\"page/js/calendar.phtml\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('GoMage_DeliveryDate')) {
            $xml .= "
<reference name=\"head\">			
    <action method=\"addItem\"><type>js</type><name>gomage/lc-calendar.js</name></action>
    <block type=\"core/html_calendar\" name=\"gomage.deliverydate.calendar\" template=\"gomage/deliverydate/js/calendar.phtml\"/>
</reference>		 
<reference name=\"checkout.onepage.shipping_method.advanced\">
    <block type=\"gomage_deliverydate/form\" name=\"checkout.onepage.shipping_method.additional.delivery_date\" template=\"gomage/deliverydate/form.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('SendCloud_Integration')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>sendcloud/onepage.js</script></action>
    <block type=\"page/html\" name=\"sendcloud_jsvalues\" template=\"sendcloud/servicepointpicker.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Symfony_Postcode')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\" ifconfig=\"symfony_postcode/settings/enabled\"><type>js</type><file>symfony/postcode.js</file></action>
    <action method=\"addItem\" ifconfig=\"symfony_postcode/settings/enabled\"><type>skin_css</type><file>css/symfony/postcode.css</file></action>
</reference>";
        }

        if ($helper->isModuleEnabled('TIG_Buckaroo3Extended')) {
            $xml .= "
<reference name=\"head\">
    <block type=\"core/template\" name=\"buckaroo_jquery\" template=\"buckaroo3extended/jquery.phtml\"/>
    <action method=\"addItem\"><type>skin_css</type><name>japi/css/TIG/Buckaroo3Extended/styles_opc.css</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/TIG/Buckaroo3Extended/paymentGuaranteeObserver.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/TIG/Buckaroo3Extended/afterpayObserver.js</name></action>
</reference>
<reference name=\"before_body_end\">
        <block type=\"core/template\" name=\"buckaroo_save_sata_js\" template=\"buckaroo3extended/saveData.phtml\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('Fooman_GoogleAnalyticsPlus')) {
            $xml .= "
<reference name=\"head\">
    <block type=\"core/text_list\" name=\"before_head_end\" as=\"before_head_end\"/>
</reference>
<reference name=\"before_head_end\">
    <block type=\"googleanalyticsplus/universal\" name=\"googleanalyticsplus_universal\" as=\"googleanalyticsplus_universal\"/>
</reference>
<reference name=\"after_body_start\">
    <block type=\"googleanalyticsplus/tagManager\" name=\"googleanalyticsplus_tagmanager\" as=\"googleanalyticsplus_tagmanager\"/>
</reference>
<reference name=\"before_body_end\">
    <block type=\"googleanalyticsplus/remarketing\" name=\"googleanalyticsplus_remarketing\" as=\"googleanalyticsplus_remarketing\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('Sevenlike_Fatturazione')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_js</type><name>js/fatturazione.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>onestepcheckout/js/autocomplete.js</name></action>
    <action method=\"addCss\"><stylesheet>onestepcheckout/autocomplete.css</stylesheet></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Bitbull_BancaSellaPro')) {
            $xml .= "
<reference name=\"head\">
    <block type=\"bitbull_bancasellapro/utility_text\" name=\"gestpay.iframe.external\"/>
    <action method=\"addJs\"><script>prototype/window.js</script></action>
    <action method=\"addItem\"><type>js_css</type><name>prototype/windows/themes/default.css</name></action>
    <action method=\"addCss\"><name>lib/prototype/windows/themes/magento.css</name></action>
    <action method=\"addJs\"><name>bancasellapro/gestpayform.js</name></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Kega_Checkout')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_js</type><name>js/kega_theme/vendor/magnific.popup.min.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/kega_checkout/main.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/kega_form/main.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/terstal_form/main.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/kega_form/address-search.js</name></action>
</reference>
<reference name=\"checkout.onepage.billing\">
    <action method=\"setTemplate\">
        <template>kega_checkout/onepage/billing.phtml</template>
    </action>
    <block type=\"checkout/onepage\" name=\"newsletter_subscription\"
        template=\"kega_checkout/onepage/newsletter_subscription.phtml\" />
    <block type=\"core/text_list\" name=\"checkout_billing_before_form\"/>
    <block type=\"core/text_list\" name=\"checkout_after_before_form\"/>
</reference>
<reference name=\"checkout.onepage.shipping_method\">
    <remove name=\"checkout.onepage.shipping_method.additional\"/>
</reference>
<reference name=\"checkout.onepage.shipping_method.available\">
    <action method=\"setTemplate\">
        <template>kega_checkout/onepage/shipping_method/available.phtml</template>
    </action>
</reference>
<reference name=\"checkout.onepage.payment\">
    <action method=\"setTemplate\">
        <template>japi/kega_checkout/onepage/payment.phtml</template>
    </action>
</reference>
<reference name=\"checkout.onepage.payment.additional\">
    <block type=\"checkout/agreements\" name=\"checkout.onepage.agreements\" as=\"additional\"
        template=\"japi/checkout/onepage/agreements.phtml\"/>
</reference>
<reference name=\"before_body_end\">
    <block type=\"kega_form/attributes\" name=\"kega.form.attributes\" template=\"kega_form/attributes.phtml\"/>
    <block type=\"kega_form/address_search\" name=\"address_search\" template=\"kega_form/address-search.phtml\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('Kega_StorePickup')) {
            $xml .= "
<reference name=\"head\">
    <block type=\"core/template\" name=\"google.maps.js\" template=\"kega_store/js/maps.phtml\"/>
    <action method=\"addItem\"><type>skin_js</type><name>js/kega_storepickup/main.js</name></action>
</reference>
<reference name=\"checkout.onepage.shipping_method.available\">
    <block type=\"storepickup/checkout_onepage_shipping_storepickup_storelist\" name=\"kega_storepickup_delivery\"
           template=\"kega_storepickup/form.phtml\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('Chronopost_Chronorelais')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_css</type><name>chronorelais/chronorelais.css</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>chronorelais/carousel-min.js</name></action>
</reference>
<reference name=\"content\">
    <block type=\"core/template\" template=\"chronorelais/checkout/onepage/shipping_method_complement.phtml\" />
</reference>";
        }

        if (Mage::getEdition() == Mage::EDITION_ENTERPRISE) {
            $xml .= "
<reference name=\"content\">
    <block type=\"core/template\" name=\"pbridge.js\" template=\"pbridge/checkout/payment/js.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Magestore_RewardPoints')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><file>magestore/rewardpoints.js</file></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Iways_PayPalPlus')) {
            $xml .= "
<reference name=\"head\">
    <block type=\"core/text\" name=\"ppplus\" ifconfig=\"payment/iways_paypalplus_payment/active\">
        <action method=\"setText\">
            <text><![CDATA[<script src=\"//www.paypalobjects.com/webstatic/ppplus/ppplus.min.js\" type=\"text/javascript\"></script>]]></text>
        </action>
    </block>
    <action method=\"addCss\" ifconfig=\"payment/iways_paypalplus_payment/active\">
        <stylesheet>css/iways-paypalplus.css</stylesheet>
    </action>
</reference>";
        }

        if ($helper->isModuleEnabled('TIG_MyParcel2014')) {
            $TIGMyParcel2014Version = $helper->getExtensionVersion('TIG_MyParcel2014');
            if ($TIGMyParcel2014Version && version_compare($TIGMyParcel2014Version, '1.17', '>=')) {
                $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/MyParcel2014/jquery-myparcel.min.js</file></action>
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/MyParcel2014/checkout.js</file></action>
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/MyParcel2014/myparcel.js</file></action>
    <action method=\"addItem\"><type>skin_css</type><file>css/TIG/MyParcel2014/myparcel_options.css</file></action>
</reference>
<reference name=\"checkout.onepage.shipping_method.available\">
    <action method=\"setTemplate\">
        <template>TIG/MyParcel2014/checkout/onepage/shipping_method/available.phtml</template>
    </action>
</reference>";
            } else {
                $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/MyParcel2014/jquery-myparcel.min.js</file></action>
    <action method=\"addItem\"><type>skin_js</type><file>js/TIG/MyParcel2014/checkout.js</file></action>
</reference>
<reference name=\"before_body_end\">
    <block type=\"core/template\" name=\"myparcel_checkout_base_url\" template=\"TIG/MyParcel2014/checkout/baseUrl.phtml\"/>
</reference>
<reference name=\"checkout.onepage.shipping_method.available\">
    <action method=\"setTemplate\">
        <template>TIG/MyParcel2014/checkout/onepage/shipping_method/available.phtml</template>
    </action>
</reference>";
            }
        }

        if ($helper->isModuleEnabled('LaPoste_SoColissimoSimplicite')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_js</type><name>js/socolissimosimplicite/shipping_method.js</name></action>
</reference>
<reference name=\"content\">
    <block type=\"japi/socolissimosimplicite_iframe\" name=\"iframe.socolissimosimplicite\" template=\"socolissimosimplicite/iframe.phtml\" after=\"checkout.onepage\" />
    <block type=\"core/template\" name=\"shippingmethod.socolissimosimplicite\" template=\"socolissimosimplicite/onepage/shipping_method/socolissimosimplicite.phtml\" after=\"iframe.socolissimosimplicite\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Borgione_General')) {
            $xml .= "
<reference name=\"checkout.onepage.billing\">
    <action method=\"setTemplate\">
        <template>persistent/checkout/onepage/billing.phtml</template>
    </action>
</reference>
<reference name=\"checkout.onepage.shipping\">
    <action method=\"setTemplate\">
        <template>checkout/onepage/shipping.phtml</template>
    </action>
</reference>";
        }

        if ($helper->isModuleEnabled('Magentix_SoColissimo')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addCss\"><stylesheet>socolissimo/css/style.css</stylesheet></action>
</reference>
<reference name=\"before_body_end\">
    <block type=\"socolissimo/frontend_socolissimo\" name=\"socolissimo\" template=\"socolissimo/socolissimo.phtml\">
        <action method=\"setData\"><name>js</name><value>socolissimo/js/checkout/magento-onepage.js</value></action>
    </block>
</reference>";
        }

        if ($helper->isModuleEnabled('DPD_Shipping')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addCss\"><stylesheet>css/dpd/shipping.css</stylesheet></action>
    <action method=\"addJs\"><script>dpd/shipping.js</script></action>
    <action method=\"addJs\"><script>prototype/window.js</script></action>
    <action method=\"addJs\"><script>dpd/window/shipping.js</script></action>
    <action method=\"addCss\"><name>css/dpd/window.css</name></action>
    <action method=\"addJs\"><file helper=\"dpd/data/getOnestepCheckoutJs\"/></action>
    <block type=\"core/template\" name=\"gmapsapi\" template=\"dpd/gmapsapi.phtml\"/>
</reference>";
        }

        if ($helper->isModuleEnabled('Flagbit_Checkout')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addCss\"><stylesheet>css/flagbit_checkout.css</stylesheet></action>
</reference>
<reference name=\"content\">
    <block type=\"page/html_wrapper\" name=\"checkout.progress.wrapper\" before=\"-\" translate=\"label\">
        <action method=\"setElementId\"><value>checkout-progress-wrapper</value></action>
        <block type=\"checkout/onepage_progress\" name=\"checkout.progress\" before=\"-\" template=\"flagbit_checkout/progress.phtml\"/>
    </block>
</reference>
<reference name=\"checkout.onepage\">
    <action method=\"setTemplate\"><template>japi/flagbit_checkout/onepage.phtml</template></action>
    <action method=\"setTemplate\" ifconfig=\"kickstarter_checkout/settings/compact_mode\"><template>japi/flagbit_checkout/onepage_compact.phtml</template></action>
</reference>
<reference name=\"checkout.onepage.billing\">
    <action method=\"setTemplate\"><template>flagbit_checkout/billing.phtml</template></action>
    <action method=\"setTemplate\" ifconfig=\"kickstarter_checkout/settings/compact_mode\"><template>flagbit_checkout/billing_compact.phtml</template></action>
    <block type=\"magesetup/checkout_agreements\" name=\"magesetup.checkout.billing.agreements\">
        <action method=\"setTemplate\" ifconfig=\"kickstarter_checkout/review/agreements_on_billing\"><template>magesetup/checkout/onepage/agreements_plain.phtml</template></action>
    </block>
    <action method=\"append\" ifconfig=\"kickstarter_checkout/settings/compact_mode\">
        <block>checkout.onepage.payment</block>
    </action>
</reference>
<reference name=\"checkout.onepage.payment\">
    <action method=\"setTemplate\"><template>checkout/onepage/payment.phtml</template></action>
</reference>";
        }

        if ($helper->isModuleEnabled('GGMGastro_CustomCheckoutFields')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>GGMGastro/CustomCheckoutFields/AdditionalFields.js</script></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/override.js</name></action>
    <action method=\"removeItem\"><type>skin_css</type><name>css/magesetup/checkout.css</name></action>
    <action method=\"addCss\"><stylesheet>css/partial/checkout.css</stylesheet><params>media=\"screen\"</params></action>
</reference>
<reference name=\"checkout.onepage.billing\">
    <block type=\"customcheckoutfields/options\" name=\"customcheckoutfields.options\">
        <action method=\"setTemplate\"><template>ggmgastro_customcheckoutfields/options.phtml</template></action>
        <block type=\"cms/block\" name=\"partial_delivery_info\" as=\"partial_delivery_info\" output=\"toHtml\">
            <action method=\"setBlockId\"><block_id>partial_delivery_info</block_id></action>
        </block>
        <block type=\"cms/block\" name=\"neutral_bill_info\" as=\"neutral_bill_info\" output=\"toHtml\">
            <action method=\"setBlockId\"><block_id>neutral_bill_info</block_id></action>
        </block>
        <block type=\"cms/block\" name=\"separate_invoice_info\" as=\"separate_invoice_info\" output=\"toHtml\">
            <action method=\"setBlockId\"><block_id>separate_invoice_info</block_id></action>
        </block>
    </block>
</reference>";
        }

        if ($helper->isModuleEnabled('MadeByMouses_Olifant')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addCss\"><stylesheet>css/olifant.css</stylesheet></action>
    <action method=\"addCss\" ifconfig=\"rewards/general/layoutsactive\"><stylesheet>css/rewards/slider.css</stylesheet></action>
    <action method=\"addCss\" ifconfig=\"rewards/general/layoutsactive\"><stylesheet>css/rewards/checkout/minicart.css</stylesheet></action>
    <action method=\"addJs\" ifconfig=\"rewards/general/layoutsactive\"><script>tbt/rewards/util.js</script></action>
    <action method=\"addJs\" ifconfig=\"rewards/general/layoutsactive\"><script>tbt/rewards/catalog/product/view/redeemer.js</script></action>
    <action method=\"addJs\" ifconfig=\"rewards/general/layoutsactive\"><script>tbt/scriptaculous/SmoothSlider.js</script></action>
    <action method=\"addJs\" ifconfig=\"rewards/general/layoutsactive\"><script>tbt/rewards/points/RedemptionSlider.js</script></action>
    <action method=\"addJs\" ifconfig=\"rewards/general/layoutsactive\"><script>tbt/rewards/catalog/product/view/PointsSlider.js</script></action>
    <action method=\"addJs\" ifconfig=\"rewards/general/layoutsactive\"><script>tbt/rewards/slider.js</script></action>
</reference>
<reference name=\"checkout.onepage\">
    <block type=\"japi/checkout_onepage_olifant\" name=\"checkout.onepage.olifant\" as=\"olifant\">
        <block type=\"core/template\" name=\"rewards_js_points_captions\" as=\"rewards_cartpoints_spender_js\" template=\"rewards/js/captions.phtml\" />
        <block type=\"rewards/checkout_cart\" name=\"rewards_points_cart_minibox\" as=\"rewards_cartpoints_spender\" template=\"rewards/checkout/cartmini.phtml\" >
            <block type=\"rewards/checkout_cart_slider\" name=\"points_cart_box.slider\" as=\"points_slider\" template=\"olifant/rewards-slider.phtml\" />
            <block type=\"core/template\" name=\"points_cart_box.js\" as=\"minicart_js\" template=\"rewards/checkout/minicart_js.phtml\" />
        </block>
    </block>
</reference>";
        }

        if (Mage::getStoreConfigFlag('japi/jmango_rest_checkout_settings/adyen_pin')
            && Mage::getStoreConfigFlag('japi/jmango_rest_checkout_settings/adyen_pin_only')) {
            $xml .= "
<reference name=\"root\">
    <action method=\"addBodyClass\"><name>adyen-pin-only</name></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Dhl_Magentolws')) {
            $xml .= "
<reference name=\"head\">
     <action method=\"addItem\"><type>js</type><name>lib/jquery/jquery-1.10.2.js</name></action>
     <action method=\"addItem\"><type>js</type><name>dhl/magentolws/venobox.min.js</name><params><![CDATA[ name=\"zjs_last\" ]]></params></action>
     <action method=\"addItem\"><type>js</type><name>dhl/magentolws/location_picker.js</name></action>
     <action method=\"addCss\"><stylesheet>css/dhl/magentolws/style.css</stylesheet></action>
</reference>";
        }

        if (Mage::getStoreConfig('afterpay_afterpay/general/active') && strpos(Mage::getUrl(), 'mostwantedluxury.com') !== false) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>moment/moment.js</script></action>
    <action method=\"addJs\"><script>pikaday/pikaday.js</script></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Ddmandcode_Routes')) {
            $xml .= "
<reference name=\"head\">
    <action  method=\"addItem\"><type>js</type><name>ddmandcode/routes/noconflict.js</name></action>
    <action  method=\"addItem\"><type>js</type><name>ddmandcode/routes/wNumb.js</name></action>
    <action  method=\"addItem\"><type>js</type><name>ddmandcode/routes/nouislider.min.js</name></action>
    <action method=\"addCss\"><stylesheet>ddmandcode/routes/nouislider.min.css</stylesheet></action>
    <action method=\"addCss\"><stylesheet>ddmandcode/routes/custom.css</stylesheet></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Mage_Authorizenet') && Mage::getStoreConfig('payment/authorizenet_directpost/active')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><file>mage/directpost.js</file></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Cryozonic_Stripe') && Mage::getStoreConfig('payment/cryozonic_stripe/active')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>skin_js</type><name>cryozonic_stripe/js/cryozonic_stripe.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>cryozonic_stripe/js/cctype.js</name></action>
    <action method=\"addCss\"><stylesheet>cryozonic_stripe/css/cctype.css</stylesheet></action>
    <action method=\"addCss\"><stylesheet>cryozonic_stripe/css/savedcards.css</stylesheet></action>
    <action method=\"addCss\"><stylesheet>cryozonic_stripe/css/paymentfont.min.css</stylesheet></action>
    <action method=\"addItem\"><type>skin_js</type><script>cryozonic_stripe/js/prototype/window.js</script></action>
    <action method=\"addItem\"><type>js_css</type><name>prototype/windows/themes/default.css</name></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Amasty_PaymentDetect')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>js</type><name>amasty/ampaymentdetect/detector.js</name></action>
    <action method=\"addCss\"><stylesheet>css/amasty/ampaymentdetect/detector.css</stylesheet></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Genmato_DirectDebitSepa')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addJs\"><script>genmato/directdebitsepa/jquery.signaturepad.min.js</script></action>
    <action method=\"addJs\"><script>genmato/directdebitsepa/flashcanvas.js</script></action>
    <action method=\"addJs\"><script>genmato/directdebitsepa/json2.min.js</script></action>
    <action method=\"addItem\"><type>skin_css</type><name>genmato/directdebitsepa/assets/jquery.signaturepad.css</name></action>
</reference>";
        }

        if ($helper->isModuleEnabled('Anowave_Ec')) {
            $xml .= "
<reference name=\"after_body_start\" after=\"-\">
    <block type=\"ec/track\" name=\"ec_noscript\" as=\"ec_noscript\" template=\"ec/noscript.phtml\"></block>
</reference>
<reference name=\"head\" after=\"-\">
    <action method=\"addItem\"><type>skin_js</type><name>js/ec.js</name></action>
    <action method=\"addItem\" ifconfig=\"ec/cookie/enable\"><type>skin_css</type><name>css/ec.css</name></action>
    <block type=\"ec/track\" name=\"ec_header\" as=\"ec_header\" template=\"ec/header.phtml\" />
    <block type=\"ec/track\" name=\"ec_purchase\" as=\"ec_purchase\" template=\"ec/purchase.phtml\">
        <block type=\"ec/track\" name=\"ec_datalayer\" as=\"ec_datalayer\" template=\"ec/datalayer.phtml\" />
        <block type=\"ec/track\" name=\"ec_visitor\" as=\"ec_visitor\" template=\"ec/visitor.phtml\" />
        <block type=\"ec/track\" name=\"ec_dimensions\" as=\"ec_dimensions\" template=\"ec/dimensions.phtml\" />
        <block type=\"ec/track\" name=\"ec_adroll\" as=\"ec_adroll\" template=\"ec/adroll.phtml\" />
        <block type=\"ec/experiments\" name=\"ec_experiments\" as=\"ec_experiments\" template=\"ec/experiments.phtml\" />
        <block type=\"ec/track\" name=\"ec_promotions\" as=\"ec_promotions\" template=\"ec/promotions.phtml\" />
        <block type=\"ec/track\" name=\"ec_impression\" as=\"ec_impression\" template=\"ec/impression.phtml\" />
        <block type=\"ec/track\" name=\"ec_details\" as=\"ec_details\" template=\"ec/details.phtml\" />
        <block type=\"ec/track\" name=\"ec_search\" as=\"ec_search\" template=\"ec/search.phtml\" />
        <block type=\"ec/track\" name=\"ec_perfectaudience\" as=\"ec_perfectaudience\" template=\"ec/perfectaudience.phtml\" />
        <block type=\"ec/track\" name=\"ec_cookie\" as=\"ec_cookie\" template=\"ec/cookie.phtml\" />
    </block>
</reference>
<reference name=\"before_body_end\" after=\"-\">
    <block type=\"ec/events\" name=\"ec_events\" as=\"ec_events\" template=\"ec/events.phtml\" />
    <block type=\"ec/track\" name=\"ec_footer\" as=\"ec_footer\" template=\"ec/footer.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Customweb_PostFinanceCw')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>js</type><name>customweb/postfinancecw/checkout.js</name><params/><if/></action>
     <action method=\"addItem\"><type>skin_css</type><name>customweb/postfinancecw/updates.css</name></action>
</reference>
<reference name=\"content\">
    <block type=\"postfinancecw/Checkout\" name=\"loadJavascriptForPostFinanceCw\" after=\"-\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Transsmart_Shipping')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addLinkRel\"><rel>stylesheet</rel><href>//fonts.googleapis.com/css?family=Dosis:500,600</href></action>
    <action method=\"addJs\"><script>transsmart/shipping/base64.js</script></action>
    <action method=\"addJs\"><script>transsmart/shipping/logger.js</script></action>
    <action method=\"addJs\"><script>transsmart/shipping/pickup_selector.js</script></action>
    <action method=\"addCss\"><style>transsmart/shipping/pickup_selector.css</style></action>
</reference>
<reference name=\"before_body_end\">
    <block type=\"transsmart_shipping/location_selector\"
           template=\"transsmart/shipping/location/selector.phtml\"
           name=\"transsmart_shipping.location.selector\" />
</reference>";
        }

        if ($helper->isModuleEnabled('Gene_Braintree')) {
            if (version_compare($helper->getExtensionVersion('Gene_Braintree'), '3.0.0', '<')) {
                $xml .= "
<reference name=\"head\">
    <block type=\"gene_braintree/assets\" name=\"gene.braintree.head.assets\" as=\"braintree.assets\" template=\"gene/braintree/assets.phtml\" after=\"-\" />
    <action method=\"addCss\"><file>css/gene/braintree/default.css</file></action>
    <action method=\"removeItem\"><type>js</type><name>braintree/braintree-1.3.4.js</name></action>
</reference>
<reference name=\"gene.braintree.head.assets\">
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/client.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/hosted-fields.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/three-d-secure.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/paypal.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/data-collector.min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/{MODULE_VERSION}/vzero-min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/{MODULE_VERSION}/vzero-paypal-min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/{MODULE_VERSION}/vzero-integration-min.js</url></action>
</reference>
<reference name=\"before_body_end\">
    <block type=\"gene_braintree/js\" name=\"gene_braintree_setup\" template=\"gene/braintree/js/setup.phtml\" />
    <remove name=\"braintree_payments_data_js\" />
    <block type=\"gene_braintree/js\" name=\"gene_braintree_js\" template=\"gene/braintree/js/default.phtml\" />
</reference>";
            } else {
                $xml .= "
<reference name=\"head\">
    <block type=\"gene_braintree/assets\" name=\"gene.braintree.head.assets\" as=\"braintree.assets\" template=\"gene/braintree/assets.phtml\" after=\"-\" />
</reference>
<reference name=\"gene.braintree.head.assets\">
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/client.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/hosted-fields.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/three-d-secure.min.js</url></action>
    <action method=\"addExternalJS\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/paypal-checkout.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/data-collector.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/apple-pay.min.js</url></action>
    <action method=\"addExternalJs\"><url>https://js.braintreegateway.com/web/{SDK_VERSION}/js/google-payment.min.js</url></action>
    <action method=\"addExternalJS\"><url>https://www.paypalobjects.com/api/checkout.js?{SDK_VERSION}</url></action>
    <action method=\"addExternalJS\"><url>https://pay.google.com/gp/p/js/pay.js?{SDK_VERSION}</url></action>
    <action method=\"addJs\"><url>gene/braintree/minified/vzero.min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/minified/vzero-paypal.min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/minified/vzero-integration.min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/minified/vzero-applepay.min.js</url></action>
    <action method=\"addJs\"><url>gene/braintree/minified/vzero-googlepay.min.js</url></action>
</reference>
<reference name=\"head\">
    <action method=\"addCss\"><file>css/gene/braintree/minified/applepay.min.css</file></action>
    <action method=\"removeItem\"><type>js</type><name>braintree/braintree-1.3.4.js</name></action>
    <action method=\"addCss\"><file>css/gene/braintree/minified/default.min.css</file></action>
</reference>
<reference name=\"before_body_end\">
    <block type=\"gene_braintree/js\" name=\"gene_braintree_setup\" template=\"gene/braintree/js/setup.phtml\" />
    <block type=\"gene_braintree/applepay_setup\" name=\"gene_braintree_applepay_setup\" template=\"gene/braintree/js/applepaysetup.phtml\" />
    <block type=\"gene_braintree/googlepay_setup\" name=\"gene_braintree_googlepay_setup\" template=\"gene/braintree/js/googlepaysetup.phtml\" />
    <remove name=\"braintree_payments_data_js\" />
    <block type=\"gene_braintree/js\" name=\"gene_braintree_js\" template=\"gene/braintree/js/default.phtml\" />
</reference>";
            }
        }

        if ($helper->isModuleEnabled('DHLParcel_Shipping')) {
            $xml .= "
<reference name=\"head\">
    <action method=\"addItem\"><type>js_css</type><name>dhlparcel/shipping/servicepoint/locator.css</name></action>
    <action method=\"addLinkRel\"><rel>stylesheet</rel><href>https://servicepoint-locator.dhlparcel.nl/servicepoint-locator.css</href></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/dhlparcel/shipping/servicepoint/locator.js</name></action>
    <action method=\"addItem\"><type>js</type><name>dhlparcel/shipping/servicepoint/locator.js</name></action>
    <action method=\"addItem\"><type>skin_js</type><name>js/dhlparcel/shipping/checkout.js</name></action>
</reference>
<reference name=\"before_body_end\">
    <block type=\"core/template\" name=\"dhlparcel_shipping_modal\" template=\"dhlparcel/shipping/modal.phtml\">
        <action method=\"setData\"><name>title</name><value>ServicePoint</value></action>
        <block type=\"core/template\" name=\"wrapper\" template=\"dhlparcel/shipping/servicepoint/wrapper.phtml\">
            <block type=\"core/template\" name=\"container\" template=\"dhlparcel/shipping/servicepoint/container.phtml\" />
        </block>
    </block>
    <block type=\"dhlparcel_shipping/jsdata_servicepoint_locator\" name=\"dhlparcel_shipping_jsdata_servicepoint\" template=\"dhlparcel/shipping/jsdata/servicepoint/locator.phtml\" />
    <block type=\"dhlparcel_shipping/jsdata_checkout\" name=\"dhlparcel_shipping_jsdata_checkout\" template=\"dhlparcel/shipping/jsdata/checkout.phtml\" />
</reference>";
        }

        if ($helper->isModuleEnabled('EasyNolo_BancaSellaPro') && Mage::getStoreConfigFlag('payment/gestpaypro/active')) {
            $xml .= "
<reference name=\"head\">
    <block type=\"easynolo_bancasellapro/utility_text\" name=\"gestpay.iframe.external\"/>
    <action method=\"addJs\"><script>prototype/window.js</script></action>
    <action method=\"addItem\"><type>skin_css</type><name>css/easynolo_bancasella/gestpay.css</name></action>
    <action method=\"addItem\"><type>js_css</type><name>prototype/windows/themes/default.css</name></action>
    <action method=\"addCss\"><name>lib/prototype/windows/themes/magento.css</name></action>
    <action method=\"addJs\" ifconfig=\"payment/gestpaypro_red/enable\"><script>bancasellapro/red_blackbox.js</script></action>
    <action method=\"addJs\"><script>bancasellapro/gestpayform.js</script></action>
    <block type=\"core/text\" name=\"paymentfont\">
        <action method=\"setText\">
            <text>
                <![CDATA[<link rel=\"stylesheet\" type=\"text/css\" href=\"https://cdnjs.cloudflare.com/ajax/libs/paymentfont/1.1.2/css/paymentfont.min.css\">]]>
            </text>
        </action>
    </block>
</reference>
<reference name=\"before_body_end\">
    <block type=\"core/text\" name=\"gestpaypro.red.script.block\">
        <action method=\"setText\" ifconfig=\"payment/gestpaypro_red/enable\">
            <text><![CDATA[<script src=\"https://mpsnare.iesnare.com/snare.js\"></script>]]></text>
        </action>
    </block>
    <block type=\"core/template\" name=\"gestpaypro.iframe.script.block\" template=\"easynolo/bancasellapro/gestpay/iframe.phtml\" ifconfig=\"payment/gestpaypro/iframe\"/>
</reference>";
        }

        try {
            $this->getLayout()->getUpdate()->addUpdate($xml);
            $this->generateLayoutXml()->generateLayoutBlocks();
        } catch (Exception $e) {
            Mage::logException($e);
        }

        $customXml = Mage::getStoreConfig('japi/jmango_rest_checkout_settings/custom_xml');
        if ($customXml) {
            try {
                $this->getLayout()->getUpdate()->addUpdate($customXml);
                $this->generateLayoutXml()->generateLayoutBlocks();
            } catch (Exception $e) {
                Mage::logException($e);
            }
        }
    }

    /**
     * Save checkout billing address
     */
    public function saveBillingAction()
    {
        Mage::app()->getStore()->setConfig('aw_mobile2/general/is_enabled', 0);
        Mage::getSingleton('core/session')->setData('is_address_update', true);
        Mage::getSingleton('core/session')->unsetData('is_billing_address_update');

        /**
         * MPLUGIN-3331 :telephone is optional on japi checkout onepage
         *
         * add load_exclude_data when Onestepcheckout is enable
         */
        if (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout')) {
            if ($billingAddress = $this->getRequest()->getPost('billing')) {
                if (is_array($billingAddress) && count($billingAddress) > 0) {
                    $helper = Mage::helper('onestepcheckout/checkout');
                    $prepareBillingData = $helper->load_exclude_data($billingAddress);
                    //ensure that address fields order is preserved after changing field order
                    if (!empty ($prepareBillingData ['street']) && is_array($prepareBillingData ['street'])) {
                        ksort($prepareBillingData ['street']);
                    }
                    if (!empty ($prepareBillingData ['street']) && is_array($prepareBillingData ['street'])) {
                        ksort($prepareBillingData ['street']);
                    }
                    if (!empty($prepareBillingData)) {
                        $this->getRequest()->setPost('billing', $prepareBillingData);
                    }
                }
            }
        }
        return parent::saveBillingAction();
    }

    /**
     * Shipping address save action
     */
    public function saveShippingAction()
    {
        Mage::app()->getStore()->setConfig('aw_mobile2/general/is_enabled', 0);
        Mage::getSingleton('core/session')->unsetData('is_shipping_address_update');

        return parent::saveShippingAction();
    }

    /**
     * Shipping method save action
     */
    public function saveShippingMethodAction()
    {
        if (Mage::helper('core')->isModuleEnabled('Wvzonline_EasyCheckout') && Mage::getStoreConfigFlag('easycheckout/config/is_enabled')) {
            $quote = $this->getOnepage()->getQuote();
            $quote->getShippingAddress()->setShippingMethod($this->getRequest()->getPost('shipping_method', ''));
            $quote->setTotalsCollectedFlag(false);
            $quote->getShippingAddress()->collectTotals();
            $quote->collectTotals();
            $quote->save();

            $result['goto_section'] = 'payment';
            $result['update_section'] = array(
                'name' => 'payment-method',
                'html' => $this->getLayout()->createBlock('easycheckout/payment')
                    ->setTemplate('easycheckout/payment/methods.phtml')
                    ->toHtml()
            );

            return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
        } elseif (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout')
            || (Mage::helper('core')->isModuleEnabled('InfoDesires_CashOnDelivery') && Mage::getStoreConfigFlag('payment/cashondelivery/active'))
        ) {
            if ($this->_expireAjax()) {
                return false;
            }
            if ($this->getRequest()->isPost()) {
                $data = $this->getRequest()->getPost('shipping_method', '');
                $result = $this->getOnepage()->saveShippingMethod($data);
                // $result will contain error data if shipping method is empty
                if (!$result) {
                    Mage::dispatchEvent(
                        'checkout_controller_onepage_save_shipping_method',
                        array(
                            'request' => $this->getRequest(),
                            'quote' => $this->getOnepage()->getQuote()
                        )
                    );
                    $this->getOnepage()->getQuote()->collectTotals();
                    $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));

                    $result['goto_section'] = 'payment';
                    $result['update_section'] = array(
                        'name' => 'payment-method',
                        'html' => $this->_getPaymentMethodsHtml()
                    );
                }
                $this->getOnepage()->getQuote()->collectTotals()->save();
                $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
            }
        } elseif (Mage::helper('core')->isModuleEnabled('Kega_Checkout')) {
            $errors = new Varien_Object();
            Mage::dispatchEvent('kega_checkout_controller_onepage_save_shipping_method', array(
                'request' => $this->getRequest(),
                'errors' => $errors
            ));
            if ($errors->getMessage()) {
                $this->getResponse()->setBody(Mage::helper('core')->jsonEncode(array(
                    'error' => -1,
                    'message' => $errors->getMessage()
                )));
            } else {
                return parent::saveShippingMethodAction();
            }
        } else {
            return parent::saveShippingMethodAction();
        }
    }

    /**
     * Save payment ajax action
     *
     * Sets either redirect or a JSON response
     */
    public function savePaymentAction()
    {
        if (Mage::helper('core')->isModuleEnabled('GGMGastro_CustomCheckoutFields')) {
            return parent::savePaymentAction();
        } else {
            if ($this->_expireAjax()) {
                return;
            }

            try {
                if (!$this->getRequest()->isPost()) {
                    $this->_ajaxRedirectResponse();
                    return;
                }

                $data = $this->getRequest()->getPost('payment', array());
                $result = $this->getOnepage()->savePayment($data);

                // get section and redirect data
                $redirectUrl = $this->getOnepage()->getQuote()->getPayment()->getCheckoutRedirectUrl();
                if (empty($result['error'])) {
                    $result['goto_section'] = 'review';
                    $result['update_section'] = array(
                        'name' => 'review',
                        'html' => $this->_getReviewHtml()
                    );
                }
                if ($redirectUrl) {
                    $result['redirect'] = $redirectUrl;
                }
            } catch (Mage_Payment_Exception $e) {
                if ($e->getFields()) {
                    $result['fields'] = $e->getFields();
                }
                $result['error'] = $e->getMessage();
            } catch (Mage_Core_Exception $e) {
                $result['error'] = $e->getMessage();
            } catch (Exception $e) {
                Mage::logException($e);
                $result['error'] = $this->__('Unable to set Payment Method.');
            }

            /* @var $helper Mage_Core_Helper_Data */
            $helper = Mage::helper('core');
            $this->getResponse()->setBody($helper->jsonEncode($result));
        }
    }

    /**
     * Get review order step html
     *
     * @return string
     * @throws
     */
    protected function _getReviewHtml()
    {
        /* @var $helper Jmango360_Japi_Helper_Data */
        $helper = Mage::helper('japi');
        $xml = '';
        if ($helper->isModuleEnabled('Mage_Authorizenet') && Mage::getStoreConfig('payment/authorizenet_directpost/active')) {
            $xml .= "
            <reference name=\"checkout.onepage.review.info.items.after\">
                <block type=\"directpost/form\" name=\"payment.form.directpost\" template=\"authorizenet/directpost/form.phtml\">
                    <action method=\"setMethodInfo\"></action>
                </block>
            </reference>";
        }

        $layout = $this->getLayout();
        $update = $layout->getUpdate();

        if (strpos(Mage::helper('core/http')->getHttpReferer(), '/japi/checkout/new') !== false) {
            $update->load('japi_checkout_new_review');
        } else {
            // load custom layout
            $update->load('japi_checkout_review');
        }

        $update->addUpdate($xml);
        $layout->generateXml();
        $layout->generateBlocks();
        $output = $layout->getOutput();
        return $output;
    }

    /**
     * Apply coupon
     */
    public function couponPostAction()
    {
        if ($this->_expireAjax()) {
            return;
        }

        if ($this->getRequest()->getParam('remove') == 1) {
            $couponCode = '';
        } else {
            $couponCode = (string)$this->getRequest()->getParam('coupon_code', '');
        }

        $quote = $this->getOnepage()->getQuote();
        $oldCouponCode = $quote->getCouponCode();
        $output = array();

        if (!strlen($couponCode) && !strlen($oldCouponCode)) {
            return;
        }

        try {
            $codeLength = strlen($couponCode);
            if (Mage::getEdition() == 'Community' && version_compare(Mage::getVersion(), '1.8.0', '>=')) {
                $isCodeLengthValid = $codeLength && $codeLength <= Mage_Checkout_Helper_Cart::COUPON_CODE_MAX_LENGTH;
            } else {
                $isCodeLengthValid = $codeLength && $codeLength <= 255;
            }

            $quote->getShippingAddress()->setCollectShippingRates(true);
            $quote->setCouponCode($isCodeLengthValid ? $couponCode : '')
                ->collectTotals()
                ->save();

            if ($codeLength) {
                if ($isCodeLengthValid && $couponCode == $quote->getCouponCode()) {
                    $output['success'] = true;
                    $output['message'] = $this->__('Coupon code "%s" was applied.', Mage::helper('core')->escapeHtml($couponCode));
                    $output['html'] = $this->_getReviewHtml();
                } else {
                    $output['success'] = false;
                    $output['message'] = $this->__('Coupon code "%s" is not valid.', Mage::helper('core')->escapeHtml($couponCode));
                }
            } else {
                $output['success'] = true;
                $output['message'] = $this->__('Coupon code was canceled.');
                $output['html'] = $this->_getReviewHtml();
            }
        } catch (Mage_Core_Exception $e) {
            $output['success'] = false;
            $output['message'] = $e->getMessage();
            Mage::logException($e);
        } catch (Exception $e) {
            $output['success'] = false;
            $output['message'] = $this->__('Cannot apply the coupon code.');
            Mage::logException($e);
        }

        /**
         * MPLUGIN-3117: Fix message with html tag
         */
        $output['message'] = strip_tags(preg_replace(
            array('/<br\/>/', '/<br>/'),
            array("\n", "\n"),
            $output['message']
        ));

        $this->getResponse()->setHeader('Content-Type', 'application/json');
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($output));
    }

    /**
     * Get shipping method step html
     *
     * @throws Mage_Core_Model_Store_Exception
     */
    public function getShippingMethodAction()
    {
        $result = array(
            'update_section' => array(
                'name' => 'shipping-method',
                'html' => $this->_getShippingMethodsHtml(),
            ),
            'goto_section' => 'shipping_method'
        );

        /**
         * MPLUGIN-2398: Support MadeByMouses_Olifant custom step
         */
        if (Mage::helper('core')->isModuleEnabled('MadeByMouses_Olifant')) {
            $isNotB2B = Mage::app()->getStore()->getCode() != 'wholesale';
            if (Mage::getSingleton('customer/session')->isLoggedIn() && $isNotB2B) {
                $result['goto_section'] = 'olifant';
                $result['allow_sections'] = array('shipping');
            }
        }

        $this->getResponse()->setHeader('Content-Type', 'application/json');
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
    }

    /**
     * Create order action
     */
    public function saveOrderAction()
    {
        $messages = array();

        /** @var Mage_Sales_Model_Quote $quote */
        $quote = $this->getOnepage()->getQuote();
        if (!$quote->getItemsCount()) {
            $messages[] = Mage::helper('japi')->__('Cart is empty.');
        }

        if (count($messages)) {
            $result['success'] = false;
            $result['error'] = true;
            $result['error_messages'] = implode("\n", $messages);
            return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
        }

        /* @var $server Jmango360_Japi_Model_Server */
        $server = Mage::getSingleton('japi/server');
        $server->setIsSubmit();
        Mage::getSingleton('core/session')->setData('jm360_check_popup', true);

        if ($requiredAgreements = Mage::helper('checkout')->getRequiredAgreementIds()) {
            if (strpos(Mage::getUrl(), 'deltastar.nl') !== false) {
                $exceptAgreementId = '4';
                $key = array_search($exceptAgreementId, $requiredAgreements);
                unset($requiredAgreements[$key]);
            }

            /**
             * MPLUGIN-3265: Support Idev_OneStepCheckout agreement
             * MPLUGIN-3440: Update
             */
            if (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout') && Mage::getStoreConfigFlag('onestepcheckout/general/rewrite_checkout_links')) {
                $oscSettings = class_exists('Idev_OneStepCheckout_Helper_Checkout') ? Mage::helper('onestepcheckout/checkout')->loadConfig() : array();
                /**
                 * Fake required agreements if default agreement disabled
                 */
                if (empty($oscSettings['enable_default_terms'])) {
                    $fakeAgreements = $this->getRequest()->getPost('agreement', array());
                    foreach ($requiredAgreements as $originId) {
                        $fakeAgreements[$originId] = 1;
                    }
                    $this->getRequest()->setPost('agreement', $fakeAgreements);
                }
                /**
                 * Add custom agreement if enabled
                 */
                if (!empty($oscSettings['enable_terms'])) {
                    $requiredAgreements[] = 'osc';
                }
            }

            $postedAgreements = array_keys($this->getRequest()->getPost('agreement', array()));
            $diff = array_diff($requiredAgreements, $postedAgreements);
            if ($diff) {
                $result['success'] = false;
                $result['error'] = true;
                $result['error_messages'] = $this->__('Please agree to all the terms and conditions before placing the order.');
                return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
            }
        }

        if ($subscribeNewsletter = $this->getRequest()->getPost('subscribe_newsletter')) {
            if (Mage::helper('core')->isModuleEnabled('Mage_Newsletter')) {
                /* @var $subscriberModel Mage_Newsletter_Model_Subscriber */
                $subscriberModel = Mage::getModel('newsletter/subscriber');
                $subscriberModel->loadByEmail($quote->getCustomerEmail());
                if (!$subscriberModel->isSubscribed()) {
                    $subscribeobj = $subscriberModel->subscribe($quote->getCustomerEmail());
                    if (is_object($subscribeobj)) {
                        $subscribeobj->save();
                    }
                }
            }
        }

        /**
         * MPLUGIN-2899: Support MEDIAMETA_QuickCheckout
         */
        if (Mage::helper('core')->isModuleEnabled('MEDIAMETA_QuickCheckout')) {
            if ($this->getRequest()->getParam('is_subscribed', false)) {
                if (!$email = Mage::getSingleton('customer/session')->getCustomer()->getEmail()) {
                    $email = $quote->getBillingAddress()->getEmail();
                }
                $subscriber = Mage::getModel('newsletter/subscriber')->loadByEmail($email);
                if (!$subscriber->getId()) {
                    $status = Mage::getModel('newsletter/subscriber')->subscribe($email);
                    if ($status == Mage_Newsletter_Model_Subscriber::STATUS_NOT_ACTIVE) {
                        //
                    }
                }
            }

            if ($heard = $this->getRequest()->getPost('quickcheckout_heard')) {
                if ($this->getRequest()->getPost('quickcheckout_other') && $heard == "Others") {
                    $heard = $this->getRequest()->getPost('quickcheckout_other');
                }
                $this->getOnepage()->getQuote()->setData('quickcheckout_heard', $heard);
            }

            if ($comment = $this->getRequest()->getPost('quickcheckout_comment')) {
                $this->getOnepage()->getQuote()->setData('quickcheckout_comment', nl2br(strip_tags($comment)));
            }
        }

        /**
         * MPLUGIN-3025: Fix order comment not saved
         */
        if (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout')) {
            if (Mage::getStoreConfig('onestepcheckout/general/rewrite_checkout_links')) {
                $enableComments = Mage::getStoreConfig('onestepcheckout/exclude_fields/enable_comments');
                $enableCommentsDefault = Mage::getStoreConfig('onestepcheckout/exclude_fields/enable_comments_default');
                $orderComment = $this->getRequest()->getPost('onestepcheckout_comments');
                $orderComment = trim($orderComment);
                if ($enableComments && !$enableCommentsDefault) {
                    if ($orderComment != "") {
                        $this->getOnepage()->getQuote()->setOnestepcheckoutCustomercomment(Mage::helper('core')->escapeHtml($orderComment));
                    }
                }
            }
        }

        if ($redirectUrl = $this->getRequest()->getPost('redirect')) {
            $result['success'] = true;
            $result['error'] = false;
            $result['redirect'] = $this->_wrapAutologin($redirectUrl);
            return $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
        }

        /**
         * Flag as JMango360 order
         */
        $this->getOnepage()->getQuote()->setData('japi', 1);

        /**
         * Clear some flags
         */
        Mage::getSingleton('core/session')->unsetData('is_address_update');

        parent::saveOrderAction();

        /**
         * MPLUGIN-2850: Wrap payment gateway url using autologin API
         */
        $body = $this->getResponse()->getBody('default');
        if ($body) {
            try {
                $body = Mage::helper('core')->jsonDecode($body);
                if (isset($body['redirect'])) {
                    /**
                     * ZOSUP-2802: Fix cart not clear in "Idev_OneStepCheckout"
                     */
                    if (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout')
                        && Mage::getStoreConfigFlag('onestepcheckout/general/rewrite_checkout_links')
                    ) {
                        $quote->setIsActive(true)->save();
                    }

                    $body['redirect'] = $this->_wrapAutologin($body['redirect']);
                    $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($body));
                }
            } catch (Exception $e) {
                $this->getResponse()->setBody($e->getMessage());
            }
        }
    }

    /**
     * Wrap an url using autologin API
     *
     * @param $url
     * @return string
     */
    protected function _wrapAutologin($url)
    {
        /** @var Jmango360_Japi_Model_Core_Cookie $cookie */
        $cookie = Mage::getSingleton('japi/core_cookie');

        return Mage::getUrl('japi/rest_mage/redirect', array(
            '_query' => array(
                'jm_id' => $cookie->get('frontend'),
                'jm_secure_id' => $cookie->get('frontend_cid'),
                'url' => $url,
                'is_submit' => 1
            ),
            '_secure' => true
        ));
    }

    /**
     * Get shipping method step html
     *
     * @return string
     * @throws
     */
    protected function _getShippingMethodsHtml()
    {
        if (strpos(Mage::helper('core/http')->getHttpReferer(), '/japi/checkout/new') !== false) {
            $layout = $this->getLayout();
            $update = $layout->getUpdate();
            $update->load('japi_checkout_new_shippingmethod');
            $layout->generateXml();
            $layout->generateBlocks();
            $output = $layout->getOutput();
            return $output;
        } else {
            if ((Mage::helper('japi')->isModuleEnabled('Netzkollektiv_InStorePickupMulti') && !Mage::helper('japi')->isModuleEnabled('TIG_MyParcel2014'))
                || strpos(Mage::getUrl(), 'divoza.com') !== false
                || Mage::getStoreConfigFlag('japi/jmango_rest_developer_settings/custom_shipping_methods')
            ) {
                $layout = $this->getLayout();
                $update = $layout->getUpdate();
                $update->load('japi_checkout_onepage_shippingmethod');
                $layout->generateXml();
                $layout->generateBlocks();
                $output = $layout->getOutput();
                return $output;
            } else {
                return parent::_getShippingMethodsHtml();
            }
        }
    }

    /**
     * Get payment method step html
     *
     * @return string
     * @throws
     */
    protected function _getPaymentMethodsHtml()
    {
        if (strpos(Mage::helper('core/http')->getHttpReferer(), '/japi/checkout/new') !== false) {
            $layout = $this->getLayout();
            $update = $layout->getUpdate();
            $update->load('japi_checkout_new_paymentmethod');
            $layout->generateXml();
            $layout->generateBlocks();
            $output = $layout->getOutput();
            return $output;
        } else {
            if (Mage::helper('core')->isModuleEnabled('Idev_OneStepCheckout')
                || (Mage::helper('core')->isModuleEnabled('InfoDesires_CashOnDelivery') && Mage::getStoreConfigFlag('payment/cashondelivery/active'))
                || Mage::getStoreConfigFlag('japi/jmango_rest_developer_settings/custom_payment_methods')
            ) {
                $layout = $this->getLayout();
                $update = $layout->getUpdate();
                $update->load('japi_checkout_onepage_paymentmethod');
                $layout->generateXml();
                $layout->generateBlocks();
                $output = $layout->getOutput();
                return $output;
            } else {
                return parent::_getPaymentMethodsHtml();
            }
        }
    }

    /**
     * Append all url need open in a popup on custom checkout page
     */
    protected function _appendPopupUrls()
    {
        $urls = explode("\n", Mage::getStoreConfig('japi/jmango_rest_checkout_settings/klarna_popup_urls'));
        if (!count($urls)) return;

        $urlsString = implode(';', $urls);
        $this->getResponse()->setHeader('Klarna-Popup-Urls', $urlsString, true);

        $head = $this->getLayout()->getBlock('head');
        if (!$head) return;

        $block = $this->getLayout()->createBlock('core/text');
        $block->setText(sprintf('<meta name="%s" content="%s">', 'Klarna-Popup-Urls', $urlsString));
        $head->append($block, 'Klarna-Popup-Urls');
    }
}
