/*
 *  jquery-boilerplate - v4.0.0
 *  A jump-start for jQuery plugins development.
 *  http://jqueryboilerplate.com
 *
 *  Made by Zeno Rocha
 *  Under MIT License
 */
!function (root, factory) {
    if (typeof define === 'function' && define.amd) {
        define([
            'jquery',
            'Jmango360_Japi/js/ladda/ladda.min',
            'Jmango360_Japi/js/prototype.noconflict',
            'https://stackpath.bootstrapcdn.com/bootstrap/4.1.1/js/bootstrap.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/select2/4.0.6-rc.0/js/select2.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/jstree/3.2.1/jstree.min.js',
            'https://cdn.jsdelivr.net/npm/emojione@3.1.4/lib/js/emojione.min.js',
            'https://cdnjs.cloudflare.com/ajax/libs/emojionearea/3.4.1/emojionearea.min.js'
        ], factory);
    } else {
        factory(root.jQuery, Ladda);
    }
}(this, function ($, Ladda, undefined) {
    "use strict";

    // undefined is used here as the undefined global variable in ECMAScript 3 is
    // mutable (ie. it can be changed by someone else). undefined isn't really being
    // passed in so we can ensure the value of it is truly undefined. In ES5, undefined
    // can no longer be modified.

    // Create the defaults once
    var pluginName = "pushMessageForm",
        defaults = {
            recipientListItemTpl: '<li class="list-group-item">' +
            ' <div class="custom-control custom-checkbox">' +
            '  <input type="checkbox" class="custom-control-input" id="{{htmlId}}" data-id="{{id}}" data-type="{{type}}" {{checked}}/>' +
            '  <label class="custom-control-label" for="{{htmlId}}">' +
            '   <i class="{{icon}}"></i> {{label}}' +
            '  </label>' +
            '  {{count}}' +
            ' </div>' +
            '</li>',
            recipientListItemCountTpl: '<span class="badge badge-primary badge-pill">{{count}}</span>',
            recipientListPagingTpl: '<li class="list-group-item">' +
            ' <div class="input-group input-group-sm">' +
            '  <div class="input-group-prepend">' +
            '   <button class="btn btn-outline-primary px-4 btn-paging" data-dir="-" data-page="{{page}}" data-limit={{limit}} {{prev_disabled}} type="button"><i class="fas fa-arrow-left"></i></button>' +
            '  </div>' +
            '  <input type="text" class="form-control text-center" value="Page {{page}} of {{max_page}}" readonly="readonly"/>' +
            '  <div class="input-group-append">' +
            '   <button class="btn btn-outline-primary px-4 btn-paging" data-dir="+" data-page="{{page}}" data-limit={{limit}} {{next_disabled}} type="button"><i class="fas fa-arrow-right"></i></button>' +
            '  </div>' +
            ' </div>' +
            '</li>',
            recipientsSelector: '#sendTo',
            addRecipientModal: '#addRecipientModal',
            sendBtnSelector: '#sendBtn',
            browseCategorySelector: '#browseCategoryBtn',
            categoryModal: '#addCategoryModal',
            categoryTreeSelector: '#category-tree',
            addCategorySelector: '#addCategoryBtn',
            deepLinkSelector: '#deepLinkTo',
            deepLinkMaxItems: 10,
            deepLinkMinInputLength: 2,
            bodySelector: '#detailMessage',
            bodyCountSelector: '#detailMessageCount',
            bodyMaxChars: 110,
            bodyCountTpl: '{{count}} characters left',
            recipientSearchMinChars: 3, // minimun recipient search query length
            recipientListMaxItems: 10, // maximun number of items in recipient list
            confirmationModal: '#confirmationModal',
            sendConfirmTitle: 'SEND PUSH MESSAGE CONFIRMATION',
            sendConfimBody: 'Push messages may be delayed by up to 2 hours due to a queuing system with Apple and Google.'
        };

    // The actual plugin constructor
    function Plugin(element, options) {
        this.element = element;

        // jQuery has an extend method which merges the contents of two or
        // more objects, storing the result in the first object. The first object
        // is generally empty as we don't want to alter the default options for
        // future instances of the plugin
        this.settings = $.extend({}, defaults, options);
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }

    // Avoid Plugin.prototype conflicts
    $.extend(Plugin.prototype, {
        init: function () {
            // Place initialization logic here
            // You already have access to the DOM element and
            // the options via the instance, e.g. this.element
            // and this.settings
            // you can add more functions like the one below and
            // call them like the example bellow

            this.recipients = {};
            this.selectedRecipients = [];
            this.tmpSelectedRecipients = [];

            this.$element = $(this.element);
            this.$recipientModal = $(this.settings.addRecipientModal);

            var self = this;

            /**
             * Listen to recipients select/unselect
             */
            this.$element.on('recipient:change', function (ev, data) {
                //self.onRecipientChange(data);
            });
            this.$element.on('tmp:recipient:change', function (ev, data) {
                self.onRecipientChange(data);
            });

            this.initForm();
            this.initRecipientModal();
            this.initCategoryModal();
            //this.initCloseForm();
        },

        /**
         * Handle iframe/window close
         */
        initCloseForm: function () {
            var self = this;
            window.addEventListener('message', function (ev) {
                if (ev.origin !== 'https://orchard.jmango360.com' &&
                    ev.origin !== 'https://orchard-uat.jmango360.com/' &&
                    ev.origin !== 'https://int-orchard.jmango360.com') {
                    return;
                }

                if (ev.data === 'close') {
                    self.closeForm();
                }
            }, false);
        },

        /**
         * Prevent closing if sending and auto logout admin session
         */
        closeForm: function () {
            if (this.isBusy()) {
                return setTimeout($.proxy(this.closeForm, this), 100);
            }

            if (!this.settings.logoutUrl) return;

            var self = this;
            $.get(this.settings.logoutUrl, null, function () {
                console.log('Logged out');
                self.signalOutside('close');
            });
        },

        /**
         * Send signal to parent window if we are in an iframe
         *
         * @param data
         */
        signalOutside: function (data) {
            if (window.parent !== window.top) { // if we are in an iframe
                window.parent.postMessage(data, self.settings.originUrl);
            }
        },

        /**
         * Init main form
         */
        initForm: function () {
            var self = this;

            /**
             * Init alerts
             */
            this.$element.find('.alert button.close').click(function () {
                $(this).parents('.alert').hide();
            });

            /**
             * Init recipients field
             */
            this.$recipients = $(this.settings.recipientsSelector);
            this.$recipients.select2({
                placeholder: this.$recipients.data('placeholder'),
                escapeMarkup: function (label) {
                    return label; // should not escape label to show icon
                }
            });
            this.$recipients.on('select2:unselect', function (ev) {
                self.unselectRecipient(ev.params.data);
            });
            this.$recipients.on('select2:select', function (ev) {
                self.selectRecipient(ev.params.data);
            });

            /**
             * Handle browse recipient button click
             */
            this.$element.on('click', '#addUserBtn', function (ev) {
                self.$recipientModal.modal('show');
            });

            /**
             * Handle browse category button click
             */
            this.$categoryModal = $(this.settings.categoryModal);
            this.$element.on('click', this.settings.browseCategorySelector, function () {
                self.$categoryModal.modal('show');
            });

            /**
             * Handle deeplink select
             */
            $(this.settings.deepLinkSelector).on('change', function () {
                var $this = $(this),
                    $targetField = $('#field' + $this.val());

                //if None
                if (!$targetField.length) {
                    var $targetFields = self.$element.find('.deep-link-target');
                    $targetFields.hide();
                    $targetFields.find('input.form-control, select.form-control').prop('disabled', true);
                } else {
                    $targetField.show();
                    $targetField.find('input.form-control, select.form-control').prop('disabled', false);
                    var $targetSiblings = $targetField.siblings('.deep-link-target');
                    $targetSiblings.hide();
                    $targetSiblings.find('input.form-control, select.form-control').prop('disabled', true);
                }
            });

            /**
             * Init deeplink to module
             */
            var $moduleInput = this.$element.find('#dl-module');
            $moduleInput.select2({
                placeholder: $moduleInput.data('placeholder')
            });
            //Prefill data
            $.get(this.settings.moduleDataUrl, null, function (data) {
                if (data.length) {
                    $moduleInput.select2({
                        data: data
                    });
                }
            });
            $moduleInput.on('select2:select', function (ev) {
                self.selectedModule = ev.params.data;
            });

            /**
             * Init deeplink to category
             */
            var $categoryInput = this.$element.find('#dl-category');
            $categoryInput.select2({
                placeholder: $categoryInput.data('placeholder'),
                minimumResultsForSearch: Infinity
            });

            /**
             * Init deeplink to product
             */
            var $productInput = this.$element.find('#dl-product');
            $productInput.select2({
                placeholder: $productInput.data('placeholder'),
                ajax: {
                    url: self.settings.productDataUrl,
                    dataType: 'json',
                    delay: 300,
                    data: function (params) {
                        return {
                            type: $productInput.data('type'),
                            query: params.term,
                            page: params.page,
                            limit: self.settings.deepLinkMaxItems
                        };
                    },
                    processResults: function (data, params) {
                        params.page = params.page || 1;

                        return {
                            results: data.items,
                            pagination: {
                                more: (params.page * self.settings.deepLinkMaxItems) < data.total
                            }
                        }
                    },
                    cache: true
                },
                minimumInputLength: self.settings.deepLinkMinInputLength,
                escapeMarkup: function (markup) {
                    return markup;
                },
                templateResult: function (data) {
                    if (data.loading) {
                        return data.text;
                    }

                    return data.name;
                },
                templateSelection: function (data) {
                    return data.name || data.text;
                }
            });
            $productInput.on('select2:select', function (ev) {
                self.selectedProduct = ev.params.data;
            });

            /**
             * Init emoji picker in body field
             */
            var bodyCountTpl = new Template(
                this.settings.bodyCountTpl,
                new RegExp('(^|.|\\r|\\n)({{\\s*(\\w+)\\s*}})', '')
            );
            var $bodyCountLabel = $(this.settings.bodyCountSelector);
            var totalChars = this.settings.bodyMaxChars;
            var $bodyTextarea = this.$element.find(this.settings.bodySelector).emojioneArea({
                autocomplete: false,
                pickerPosition: 'bottom',
                filtersPosition: "bottom",
                search: false,
                tonesStyle: 'square',
                events: {
                    keydown: function (editor, e) {
                        var thisChars = emojioneArea.getText().length;
                        if (self.checkKeyChar(e) && thisChars >= totalChars) {
                            e.stopImmediatePropagation();
                            e.preventDefault();
                            e.stopPropagation();
                        }
                    },
                    keyup: function () {
                        trimText();
                    },
                    emojibtn_click: function () {
                        trimText(true);
                    },
                    paste: function () {
                        trimText();
                    }
                }
            });
            var emojioneArea = $bodyTextarea[0].emojioneArea;
            var currentTxt = "";
            var trimText = function (isEmoClick) {
                //var emoCat = $(".emojionearea-category");
                var thisChars = emojioneArea.getText().length;
                if (thisChars + 2 > totalChars) {
                    //emoCat.addClass("not-active");
                    if (thisChars > totalChars && isEmoClick) {
                        emojioneArea.setText(currentTxt);
                        setFocus();
                        //emoCat.removeClass("not-active");
                        $bodyCountLabel.html(bodyCountTpl.evaluate({
                            count: Math.max(totalChars - $bodyTextarea.val().length, 0)
                        }));
                        return;
                    } else if (thisChars > totalChars) {
                        emojioneArea.setText(emojioneArea.getText().substr(0, totalChars));
                        setFocus();
                    }
                } else {
                    //emoCat.removeClass("not-active");
                }
                currentTxt = emojioneArea.getText();
                $bodyCountLabel.html(bodyCountTpl.evaluate({count: Math.max(totalChars - thisChars, 0)}));
            };
            var setFocus = function () {
                var el = $(".emojionearea-editor")[0];
                el.focus();
                if (typeof window.getSelection !== "undefined" && typeof document.createRange !== "undefined") {
                    var range = document.createRange();
                    range.selectNodeContents(el);
                    range.collapse(false);
                    var sel = window.getSelection();
                    sel.removeAllRanges();
                    sel.addRange(range);
                } else if (typeof document.body.createTextRange !== "undefined") {
                    var textRange = document.body.createTextRange();
                    textRange.moveToElementText(el);
                    textRange.collapse(false);
                    textRange.select();
                }
            };
            var updateCountLabel = function () {
                if (emojioneArea.isReady) {
                    $bodyCountLabel.html(bodyCountTpl.evaluate({
                        count: Math.max(totalChars - emojioneArea.getText().length, 0)
                    }));
                } else {
                    setTimeout(function () {
                        updateCountLabel();
                    }, 100);
                }
            };
            updateCountLabel();

            /**
             * Init Send button (Ladda)
             */
            this.sendLadda = Ladda.create($(this.settings.sendBtnSelector).get(0));

            /**
             * Handle Send button click
             */
            this.$element.on('click', this.settings.sendBtnSelector, function () {
                self.$element.submit();
            });

            /**
             * Bind form submit event to validate form
             */
            this.$element.submit(function (event) {
                var $this = $(this);
                $this.addClass('was-validated');

                var ok = true;
                if (this.checkValidity() === false) {
                    ok = false;
                }

                event.preventDefault();
                event.stopPropagation();

                if (ok) {
                    self.showConfirm({
                        title: self.settings.sendConfirmTitle,
                        body: self.settings.sendConfimBody,
                        ok: function () {
                            //show busy
                            self.sendLadda.start();

                            //prepare params
                            var params = self.$element.serializeArray();

                            for (var i = 0; i < self.selectedRecipients.length; i++) {
                                var tmp = self.selectedRecipients[i];
                                params.push({
                                    name: 'recipient[' + tmp.type + '][]',
                                    value: tmp.oid
                                });
                            }

                            switch (self.$element.find('#deepLinkTo').val()) {
                                case '1': //category
                                    if (self.selectedCategory) {
                                        params.push({
                                            name: 'moduleType',
                                            value: self.selectedCategory.otype
                                        });
                                        params.push({
                                            name: 'moduleId',
                                            value: self.selectedCategory.id
                                        });
                                        params.push({
                                            name: 'moduleName',
                                            value: self.selectedCategory.text
                                        });
                                    }
                                    break;
                                case '2': //module
                                    if (self.selectedModule) {
                                        params.push({
                                            name: 'moduleType',
                                            value: self.selectedModule.otype
                                        });
                                        params.push({
                                            name: 'moduleId',
                                            value: self.selectedModule.id
                                        });
                                        params.push({
                                            name: 'moduleName',
                                            value: self.selectedModule.text
                                        });
                                    }
                                    break;
                                case '3': //product
                                    if (self.selectedProduct) {
                                        params.push({
                                            name: 'productUrl',
                                            value: self.selectedProduct.url
                                        });
                                        params.push({
                                            name: 'productId',
                                            value: self.selectedProduct.id
                                        });
                                    }
                                    break;
                            }

                            //send
                            self.isSendingPushMessage = true;
                            self.hideAllMessages();
                            $.post(self.settings.sendUrl, params, function (data) {
                                if (data.ok === false || data.error === 1) {
                                    self.showErrorMessage(data.message)
                                } else {
                                    self.showSuccessMessage('Success!');
                                    self.signalOutside('close');
                                }
                            }).always(function () {
                                //hide busy
                                self.sendLadda.stop();
                                self.isSendingPushMessage = false;
                            });
                        }
                    });
                }
            });
        },

        checkKeyChar: function (e) {
            var keyChar = e.which > 0 && !e.ctrlKey && !e.metaKey && !e.altKey && e.which != 8 && e.which != 46 && (e.which < 37 || e.which > 40);
            return typeof e.which != "undefined" && keyChar;
        },

        /**
         * Show a success message
         *
         * @param message
         */
        showSuccessMessage: function (message) {
            this.$element.find('.alert-success p').html(message);
            this.$element.find('.alert-success').show();
        },

        /**
         * Show a error message
         *
         * @param message
         */
        showErrorMessage: function (message) {
            this.$element.find('.alert-danger p').html(message);
            this.$element.find('.alert-danger').show();
        },

        /**
         * Hide all messages
         */
        hideAllMessages: function () {
            this.$element.find('.alert').hide();
        },

        /**
         * Check if push message is sending
         */
        isBusy: function () {
            return this.isSendingPushMessage;
        },

        /**
         * Init recipient modal
         */
        initRecipientModal: function () {
            this.recipientListItemTpl = new Template(
                this.settings.recipientListItemTpl,
                new RegExp('(^|.|\\r|\\n)({{\\s*(\\w+)\\s*}})', '')
            );
            this.recipientListPagingTpl = new Template(
                this.settings.recipientListPagingTpl,
                new RegExp('(^|.|\\r|\\n)({{\\s*(\\w+)\\s*}})', '')
            );
            this.recipientListItemCountTpl = new Template(
                this.settings.recipientListItemCountTpl,
                new RegExp('(^|.|\\r|\\n)({{\\s*(\\w+)\\s*}})', '')
            );
            this.$selectedRecipent = this.$recipientModal.find('#selectedRecipients');

            var self = this;

            /**
             * Show first tab when modal open
             */
            this.$recipientModal.on('show.bs.modal', function (ev) {
                var id = self.$recipientModal.find('#groupTabs button.active').data('id');
                if (id) {
                    self.showTabContent(id);
                }
            });

            /**
             * Handle recipient tabs click
             */
            this.$recipientModal.on('click', '.btn-group-tab', function (ev) {
                if ($(this).hasClass('active')) return;

                //set active button
                $(this).addClass('active').siblings('button').removeClass('active');


                //show tab content
                var id = $(this).data('id');
                if (id) {
                    self.showTabContent(id)
                }
            });

            /**
             * Handle recipient select
             */
            this.$recipientModal.on('change', '.custom-control-input', function (ev) {
                var $this = $(this),
                    checked = this.checked,
                    checkAll = $this.data('check') === 'all';

                if (checkAll) {
                    var parent = $this.parents('li');
                    if (parent.length) {
                        parent.siblings('li').find('.custom-control-input')
                            .prop('checked', checked)
                            .trigger('change'); //TODO: switch to triggerHandler to prevent event bubbling
                    }
                } else {
                    var id = $this.data('id'),
                        type = $this.data('type');

                    if (self.recipients.hasOwnProperty(type)) {
                        var data = self.recipients[type];
                        for (var i = 0; i < data.length; i++) {
                            if (data[i]['id'] === id) {
                                if (checked) {
                                    data[i].selected = true;
                                    //self.selectRecipient(data[i]);
                                    self.selectRecipientTmp(data[i]);
                                } else {
                                    data[i].selected = false;
                                    //self.unselectRecipient(data[i]);
                                    self.unselectRecipientTmp(data[i]);
                                }
                            }
                        }
                    }
                }
            });

            /**
             * Handle search field
             */
            this.$recipientModal.on('change keyup', '.form-search', function () {
                var $this = $(this),
                    query = $this.val();

                if (query.length > 0 && query.length < self.settings.recipientSearchMinChars) return;
                if (query === $this.data('query')) return;
                $this.data('query', query);

                var timer;
                if (timer = $this.data('timer')) {
                    clearTimeout(timer);
                }

                timer = setTimeout(function () {
                    var $container = $this.parents('.list-group-content'),
                        type = $container.data('type');

                    self.loadContent({
                        id: type,
                        page: 1,
                        limit: self.settings.recipientListMaxItems,
                        query: query
                    }, $container);
                }, 300);
                $this.data('timer', timer);
            });

            /**
             * Handle search field clear button
             */
            this.$recipientModal.on('click', '.list-group-item-search .btn-clear', function () {
                var $search = $(this).parents('.list-group-item-search').find('.form-search');

                if ($search.length) $search.val('').trigger('change');
            });

            /**
             * Handle prev/next navigation in recipient list
             */
            this.$recipientModal.on('click', '.btn-paging', function () {
                var $this = $(this),
                    dir = $this.data('dir'),
                    page = $this.data('page'),
                    limit = $this.data('limit'),
                    $container = $this.parents('.list-group-content'),
                    query = $container.find('.list-group-item-search input').val(),
                    type = $container.data('type');

                if (type && page > 1 && dir === '-') {
                    $this.prop('disabled', 'disabled');
                    self.loadContent({id: type, limit: limit, page: page - 1, query: query}, $container);
                } else if (type && page >= 1 && dir === '+') {
                    $this.prop('disabled', 'disabled');
                    self.loadContent({id: type, limit: limit, page: page + 1, query: query}, $container);
                }
            });

            /**
             * Handle add selected recipients from modal to form
             */
            this.$recipientModal.on('click', '#selectBtn', function () {
                self.addRecipients();
                self.$recipientModal.modal('hide');
            });

            this.$recipientModal.on('hide.bs.modal', function () {
                self.tmpSelectedRecipients = [];
            });

            /**
             * Render selected list on modal show
             */
            this.$recipientModal.on('show.bs.modal', function () {
                //Clear tmp selected list
                self.tmpSelectedRecipients = self.selectedRecipients.slice();

                //Uncheck origin list
                self.$recipientModal.find('.list-group-item input[type="checkbox"]')
                    .prop('checked', false);

                //Fill selected list
                var html = [];
                for (var i = 0; i < self.selectedRecipients.length; i++) {
                    var data = self.selectedRecipients[i];
                    html.push(self.recipientListItemTpl.evaluate({
                        htmlId: self.getSelectedRecipientHtmlId(data),
                        type: data['type'],
                        id: data['id'],
                        icon: data['icon'],
                        label: data['label'],
                        checked: 'checked',
                        count: data['count'] ? self.recipientListItemCountTpl.evaluate({count: data['count']}) : ''
                    }));

                    var originItem = self.$recipientModal.find('#' + self.getRecipientHtmlId(data));
                    if (originItem.length) {
                        originItem.prop('checked', true);
                    }
                }
                self.$selectedRecipent.html(html.join(''));
            });
        },

        /**
         * Get data
         */
        getDataByTypeAndId: function (type, id) {
            if (!id || !type) return;
            if (this.recipients.hasOwnProperty(type)) {
                var data = this.recipients[type];
                for (var i = 0; i < data.length; i++) {
                    if (data[i]['id'] === id) {
                        return data[i];
                    }
                }
            }
        },

        /**
         * Init category modal
         */
        initCategoryModal: function () {
            var self = this;

            /**
             * Init category tree
             */
            this.$categoryTree = $(this.settings.categoryTreeSelector);
            this.categoryTree = this.$categoryTree.jstree({
                core: {
                    animation: false,
                    multiple: false,
                    check_callback: true,
                    data: {
                        url: function (node) {
                            return self.settings.categoryDataUrl;
                        },
                        data: function (node) {
                            return {id: node.id};
                        }
                    }
                },
                plugins: ['search']
            }).jstree();

            /**
             * Handle Add Category button
             */
            this.$categoryModal.on('click', this.settings.addCategorySelector, function () {
                var selectedNodes = self.categoryTree.get_selected(true);
                if (selectedNodes.length) {
                    self.addCategory(selectedNodes);
                    self.$categoryModal.modal('hide');
                } else {
                    self.showError(self.__('Please select a category!'));
                }
            });
        },

        /**
         * Show confirmation modal
         *
         * @param config
         */
        showConfirm: function (config) {
            if (!this.$confirmModal)
                this.$confirmModal = $(this.settings.confirmationModal);
            if (!this.$confirmModal.length) return;

            this.$confirmModal.find('.modal-title').html(config.title || '');
            this.$confirmModal.find('.modal-body p').html(config.body || '');
            this.$confirmModal.off('click', '.btn-ok', $.proxy(this.onConfirmModalOk, this, config, this.$confirmModal));
            this.$confirmModal.on('click', '.btn-ok', $.proxy(this.onConfirmModalOk, this, config, this.$confirmModal));

            this.$confirmModal.modal('show');
        },

        /**
         * Confirmation modal YES callback
         *
         * @param config
         * @param $modal
         */
        onConfirmModalOk: function (config, $modal) {
            $modal.modal('hide');

            if ($.isFunction(config.ok)) {
                config.ok.call(this);
            }
        },

        /**
         * Show an error message to user
         *
         * @param msg
         */
        showError: function (msg) {
            alert(msg);
        },

        /**
         * Wrapper for translate function
         *
         * @param str
         */
        __: function (str) {
            if (typeof Translator !== 'undefined') {
                return Translator.translate(str);
            } else {
                return str;
            }
        },

        /**
         * Add selected category to main form
         *
         * @param nodes
         */
        addCategory: function (nodes) {
            if (!nodes.length) return;
            var node = nodes[0];
            var html = new Option(node.text, node.id, false, true);
            var $select = $('#dl-category', this.$element);
            $select.empty();
            $select.append(html);
            $select.triggerHandler('change');
            this.selectedCategory = node.original;
        },

        /**
         * Render selected recipients to main form
         */
        addRecipients: function () {
            //copy tmp array
            this.selectedRecipients = this.tmpSelectedRecipients.slice();

            //clear all items
            this.$recipients.empty();

            //apply new items
            this.$recipients.select2({
                data: this.selectedRecipients,
                escapeMarkup: function (label) {
                    return label; // should not escape label to show icon
                }
            });

            //auto select new items
            this.$recipients
                .val(this.selectedRecipients.map(function (o) {
                    return o.id
                }))
                .triggerHandler('change');
        },

        /**
         * Render selected recipients list in modal
         */
        onRecipientChange: function (data) {
            //populate new selected list
            if (data.selected) {
                this.$selectedRecipent.append(this.recipientListItemTpl.evaluate({
                    htmlId: this.getSelectedRecipientHtmlId(data),
                    type: data['type'],
                    id: data['id'],
                    icon: data['icon'],
                    label: data['label'],
                    checked: 'checked',
                    count: data['count'] ? this.recipientListItemCountTpl.evaluate({count: data['count']}) : ''
                }));
            } else {
                var selectedItem = this.$selectedRecipent.find('#' + this.getSelectedRecipientHtmlId(data));
                if (selectedItem.length) {
                    selectedItem.parents('.list-group-item').remove();
                }
                var originItem = this.$recipientModal.find('#' + this.getRecipientHtmlId(data));
                if (originItem.length)
                    originItem.prop('checked', false);
            }
        },

        /**
         * Get html ID for checkbox on origin list
         *
         * @param item
         */
        getRecipientHtmlId: function (item) {
            return item ? 'r_' + item['type'] + '_' + item['id'] : '';
        },

        /**
         * Get html ID for checkbox on selected list
         *
         * @param item
         */
        getSelectedRecipientHtmlId: function (item) {
            return item ? 'sr_' + item['type'] + '_' + item['id'] : '';
        },

        /**
         * Remove recipient from selected list
         *
         * @param recipient
         */
        unselectRecipient: function (recipient) {
            var i = this.indexOfSelectedRecipient(recipient);
            if (i !== -1) {
                this.selectedRecipients.splice(i, 1);
                this.$element.triggerHandler('recipient:change', recipient);
            }
        },

        /**
         * Remove recipient from tmp selected list
         *
         * @param recipient
         */
        unselectRecipientTmp: function (recipient) {
            var i = this.indexOfSelectedRecipient(recipient, this.tmpSelectedRecipients);
            if (i !== -1) {
                this.tmpSelectedRecipients.splice(i, 1);
                this.$element.triggerHandler('tmp:recipient:change', recipient);
            }
        },

        /**
         * Add recipient to selected list
         *
         * @param recipient
         */
        selectRecipient: function (recipient) {
            if (this.indexOfSelectedRecipient(recipient) === -1) {
                this.selectedRecipients.push(recipient);
                this.$element.triggerHandler('recipient:change', recipient);
            }
        },

        /**
         * Add recipient to tmp selected list
         *
         * @param recipient
         */
        selectRecipientTmp: function (recipient) {
            if (this.indexOfSelectedRecipient(recipient, this.tmpSelectedRecipients) === -1) {
                this.tmpSelectedRecipients.push(recipient);
                this.$element.triggerHandler('tmp:recipient:change', recipient);
            }
        },

        /**
         * Check index of 'recipient' in 'selectedRecipients' array
         * Return index if found, otherwise return -1.
         *
         * @param recipient
         * @returns {number}
         */
        indexOfSelectedRecipient: function (recipient, recipients) {
            recipients = recipients || this.selectedRecipients;
            for (var i = 0; i < recipients.length; i++) {
                var tmp = recipients[i];
                if (tmp.type == recipient.type && tmp.id == recipient.id) {
                    return i;
                }
            }
            return -1;
        },

        /**
         * Check index of 'recipient' in 'recipients' array
         * Return index if found, otherwise return -1.
         *
         * @param recipient
         * @returns {number}
         */
        indexOfRecipient: function (recipient) {
            var type = recipient.type;
            if (this.recipients[type]) {
                for (var i = 0; i < this.recipients[type].length; i++) {
                    var tmp = this.recipients[type][i];
                    if (tmp.type == recipient.type && tmp.id == recipient.id) {
                        return i;
                    }
                }
            }

            return -1;
        },

        /**
         * Show tab content with check data loaded flag
         *
         * @param id
         */
        showTabContent: function (id) {
            var $content = this.$recipientModal.find('#groupList' + id);
            if ($content.length) {
                if (id == 'USERS') {
                    $content.find('li.list-group-item-checkall').hide();
                    $content.find('li.list-group-item-search').show();
                }
                $content.show().siblings('.list-group').hide();
                if ($content.data('loaded') != '1') {
                    this.loadContent({id: id, limit: this.settings.recipientListMaxItems, page: 1}, $content);
                }
            }
        },

        /**
         * Load list from server and append to container
         *
         * @param params
         * @param $container
         */
        loadContent: function (params, $container) {
            var self = this,
                id = params.id,
                $loader = $container.find('.list-group-item-loading'),
                $list = $container.find('ul.list-group');

            if (!this.recipients[id]) {
                this.recipients[id] = [];
            }

            $loader.show();

            $.getJSON(this.settings.recipientDataUrl, params, function (data) {
                //hide loading indicator
                $loader.hide();

                //set loaded flag
                $container.data('loaded', 1);

                //remove all previous generated items
                $list.empty();

                //convert and store data
                if (data.options) {
                    for (var i = 0; i < data.options.length; i++) {
                        data.options[i].type = data.type;
                        data.options[i].icon = data.icon;
                        data.options[i].text = '<i class="' + data.icon + '"></i> ' + data.options[i].label;
                        data.options[i].title = data.options[i].label;

                        if (self.indexOfRecipient(data.options[i]) === -1) {
                            self.recipients[id].push(data.options[i]);
                        }
                    }

                    var html = [];

                    //populate list html
                    for (var i = 0; i < data.options.length; i++) {
                        if (data.options.hasOwnProperty(i)) {
                            html.push(self.recipientListItemTpl.evaluate({
                                htmlId: self.getRecipientHtmlId(data.options[i]),
                                type: id,
                                id: data.options[i]['id'],
                                icon: data.options[i]['icon'],
                                label: data.options[i]['label'],
                                checked: self.indexOfSelectedRecipient(data.options[i], self.tmpSelectedRecipients) !== -1 ? 'checked' : '',
                                count: data.options[i]['count'] ? self.recipientListItemCountTpl.evaluate({count: data.options[i]['count']}) : ''
                            }));
                        }
                    }

                    //insert paging navigation if list too long
                    if (data.limit && data.page && data.total) {
                        var maxPage = Math.ceil(data.total / data.limit);
                        if (data.page <= maxPage) {
                            html.push(self.recipientListPagingTpl.evaluate({
                                page: data.page,
                                max_page: maxPage,
                                limit: data.limit,
                                prev_disabled: data.page == 1 ? 'disabled' : '',
                                next_disabled: data.page == maxPage ? 'disabled' : ''
                            }));
                        }
                    }

                    self.replaceHtml($list, html.join(''));
                } else {
                    self.recipients[id] = [];
                }
            });
        },

        /**
         * Fast html replacement using DocumentFragment
         * TODO: It seem not fast because of "document.createElement()"
         *
         * @param $container
         * @param html
         */
        replaceHtml: function ($container, html) {
            if (!$container.length) return;

            var fragment = document.createDocumentFragment(),
                tmpDoc = document.createElement('body');

            tmpDoc.innerHTML = html;
            while (tmpDoc.firstChild) {
                fragment.appendChild(tmpDoc.firstChild);
            }

            var dom = $container.get(0);
            dom.innerHTML = '';
            dom.appendChild(fragment);
        }
    });

    // A really lightweight plugin wrapper around the constructor,
    // preventing against multiple instantiations
    $.fn[pluginName] = function (options) {
        return this.each(function () {
            if (!$.data(this, "plugin_" + pluginName)) {
                $.data(this, "plugin_" + pluginName, new Plugin(this, options));
            }
        });
    };
});
