<?php
/**
 * Copyright 2019 JMango360
 */

/**
 * Class Jmango360_Japi_Model_Core_Session
 * Override 'core/session' with namespace 'frontend' (the first session to be initialized)
 */
class Jmango360_Japi_Model_Core_Session extends Mage_Core_Model_Session
{
    /**
     * Pre-defined session namespaces
     *
     * @var array
     */
    protected $_namespaces = array(
        Mage_Core_Model_Session_Abstract_Varien::SECURE_COOKIE_CHECK_KEY,
        'catalog',
        'catalogsearch',
        'checkout',
        'reports',
        'rss',
        'admin',
        'adminhtml_quote'
    );

    protected $_bypassUrls = array(
        '/japi/rest_mage/getSession/',
        '/japi/rest_customer/login/'
    );


    /**
     * Validate session
     */
    public function validate()
    {
        $this->_fakeSessionValidatorData();

        return parent::validate();
    }

    /**
     * Reset validator data in $_SESSION for every namespaces
     * @throws
     */
    protected function _fakeSessionValidatorData()
    {
        /**
         * Current object is core/session
         */
        $this->setData(self::VALIDATOR_KEY, $this->getValidatorData());

        /**
         * Pre-init all available session namespaces on website.
         * If we missed one, the request will be die.
         */
        /** @var Jmango360_Japi_Model_Core_Dummy $model */
        $model = Mage::getModel('japi/core_dummy');
        /** @var Jmango360_Japi_Helper_Serialize $md5Helper */
        $md5Helper = Mage::helper('japi/serialize');
        $secureCookie = Mage::app()->getFrontController()->getRequest()->getParam('jm_secure_id');
        $names = $this->_getSessionNamespaces();
        foreach ($names as $name) {
            if (!$name) continue;

            $model = $model->init($name);
            if ($model) {
                /**
                 * Try to override '_secure_cookie_check' with any value.
                 * There are no '_secure_cookie_check' session namespace in Magento,
                 * but we still include here because we try to exploit it via protected property '_data' of Varien_Object.
                 * Here the code we exploit:
                 *
                 * Function: Mage_Core_Model_Session_Abstract_Varien->init()
                 * $this->_data = &$_SESSION[$namespace];
                 *
                 * Function: Mage_Core_Model_Session_Abstract_Varien->start()
                 * $_SESSION[self::SECURE_COOKIE_CHECK_KEY] = md5($checkId);
                 */
                if ($name == Mage_Core_Model_Session_Abstract_Varien::SECURE_COOKIE_CHECK_KEY) {
                    if ($secureCookie) {
                        $refModel = new ReflectionObject($model);
                        $refProperty = $refModel->getProperty('_data');
                        $refProperty->setAccessible(true);
                        $refProperty->setValue($model, $md5Helper->md5($secureCookie));
                    }
                } else {
                    $model->setData(self::VALIDATOR_KEY, $this->getValidatorData());
                }
            }
        }
    }

    /**
     * Collect all session namespace
     *
     * @return array
     * @throws Mage_Core_Model_Store_Exception
     */
    protected function _getSessionNamespaces()
    {
        $names = explode("\n", Mage::getStoreConfig('japi/jmango_rest_developer_settings/include_sessions'));
        $names = array_merge($names, $this->_namespaces);

        /**
         * Customer session
         */
        $customerSession = 'customer';
        if (Mage::getSingleton('customer/config_share')->isWebsiteScope()) {
            $customerSession .= '_' . Mage::app()->getStore()->getWebsite()->getCode();
        }

        $names[] = $customerSession;

        /**
         * Store session
         */
        foreach (Mage::app()->getStores() as $store) {
            /** @var Mage_Core_Model_Store $store */
            $names[] = 'store_' . $store->getCode();
        }

        return $names;
    }

    /**
     * @return Jmango360_Japi_Model_Core_Cookie
     */
    public function getCookie()
    {
        if (in_array(Mage::app()->getFrontController()->getRequest()->getRequestString(), $this->_bypassUrls)) {
            return Mage::getSingleton('core/cookie');
        } else {
            return Mage::getSingleton('japi/core_cookie');
        }
    }

    /**
     * Check whether SID can be used for session initialization
     * Admin area will always have this feature enabled
     * Always enabled with 'japi' routers
     *
     * @return bool
     */
    public function useSid()
    {
        return true;
    }
}
